"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.duration_pb2
import google.protobuf.empty_pb2
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import google.protobuf.timestamp_pb2
import shared_pb2
import status_pb2
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class ReadReq(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class Options(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        class _ReadDirection:
            ValueType = typing.NewType("ValueType", builtins.int)
            V: typing_extensions.TypeAlias = ValueType

        class _ReadDirectionEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[ReadReq.Options._ReadDirection.ValueType], builtins.type):  # noqa: F821
            DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
            Forwards: ReadReq.Options._ReadDirection.ValueType  # 0
            Backwards: ReadReq.Options._ReadDirection.ValueType  # 1

        class ReadDirection(_ReadDirection, metaclass=_ReadDirectionEnumTypeWrapper): ...
        Forwards: ReadReq.Options.ReadDirection.ValueType  # 0
        Backwards: ReadReq.Options.ReadDirection.ValueType  # 1

        class StreamOptions(google.protobuf.message.Message):
            DESCRIPTOR: google.protobuf.descriptor.Descriptor

            STREAM_IDENTIFIER_FIELD_NUMBER: builtins.int
            REVISION_FIELD_NUMBER: builtins.int
            START_FIELD_NUMBER: builtins.int
            END_FIELD_NUMBER: builtins.int
            @property
            def stream_identifier(self) -> shared_pb2.StreamIdentifier: ...
            revision: builtins.int
            @property
            def start(self) -> shared_pb2.Empty: ...
            @property
            def end(self) -> shared_pb2.Empty: ...
            def __init__(
                self,
                *,
                stream_identifier: shared_pb2.StreamIdentifier | None = ...,
                revision: builtins.int = ...,
                start: shared_pb2.Empty | None = ...,
                end: shared_pb2.Empty | None = ...,
            ) -> None: ...
            def HasField(self, field_name: typing_extensions.Literal["end", b"end", "revision", b"revision", "revision_option", b"revision_option", "start", b"start", "stream_identifier", b"stream_identifier"]) -> builtins.bool: ...
            def ClearField(self, field_name: typing_extensions.Literal["end", b"end", "revision", b"revision", "revision_option", b"revision_option", "start", b"start", "stream_identifier", b"stream_identifier"]) -> None: ...
            def WhichOneof(self, oneof_group: typing_extensions.Literal["revision_option", b"revision_option"]) -> typing_extensions.Literal["revision", "start", "end"] | None: ...

        class AllOptions(google.protobuf.message.Message):
            DESCRIPTOR: google.protobuf.descriptor.Descriptor

            POSITION_FIELD_NUMBER: builtins.int
            START_FIELD_NUMBER: builtins.int
            END_FIELD_NUMBER: builtins.int
            @property
            def position(self) -> global___ReadReq.Options.Position: ...
            @property
            def start(self) -> shared_pb2.Empty: ...
            @property
            def end(self) -> shared_pb2.Empty: ...
            def __init__(
                self,
                *,
                position: global___ReadReq.Options.Position | None = ...,
                start: shared_pb2.Empty | None = ...,
                end: shared_pb2.Empty | None = ...,
            ) -> None: ...
            def HasField(self, field_name: typing_extensions.Literal["all_option", b"all_option", "end", b"end", "position", b"position", "start", b"start"]) -> builtins.bool: ...
            def ClearField(self, field_name: typing_extensions.Literal["all_option", b"all_option", "end", b"end", "position", b"position", "start", b"start"]) -> None: ...
            def WhichOneof(self, oneof_group: typing_extensions.Literal["all_option", b"all_option"]) -> typing_extensions.Literal["position", "start", "end"] | None: ...

        class SubscriptionOptions(google.protobuf.message.Message):
            DESCRIPTOR: google.protobuf.descriptor.Descriptor

            def __init__(
                self,
            ) -> None: ...

        class Position(google.protobuf.message.Message):
            DESCRIPTOR: google.protobuf.descriptor.Descriptor

            COMMIT_POSITION_FIELD_NUMBER: builtins.int
            PREPARE_POSITION_FIELD_NUMBER: builtins.int
            commit_position: builtins.int
            prepare_position: builtins.int
            def __init__(
                self,
                *,
                commit_position: builtins.int = ...,
                prepare_position: builtins.int = ...,
            ) -> None: ...
            def ClearField(self, field_name: typing_extensions.Literal["commit_position", b"commit_position", "prepare_position", b"prepare_position"]) -> None: ...

        class FilterOptions(google.protobuf.message.Message):
            DESCRIPTOR: google.protobuf.descriptor.Descriptor

            class Expression(google.protobuf.message.Message):
                DESCRIPTOR: google.protobuf.descriptor.Descriptor

                REGEX_FIELD_NUMBER: builtins.int
                PREFIX_FIELD_NUMBER: builtins.int
                regex: builtins.str
                @property
                def prefix(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
                def __init__(
                    self,
                    *,
                    regex: builtins.str = ...,
                    prefix: collections.abc.Iterable[builtins.str] | None = ...,
                ) -> None: ...
                def ClearField(self, field_name: typing_extensions.Literal["prefix", b"prefix", "regex", b"regex"]) -> None: ...

            STREAM_IDENTIFIER_FIELD_NUMBER: builtins.int
            EVENT_TYPE_FIELD_NUMBER: builtins.int
            MAX_FIELD_NUMBER: builtins.int
            COUNT_FIELD_NUMBER: builtins.int
            CHECKPOINTINTERVALMULTIPLIER_FIELD_NUMBER: builtins.int
            @property
            def stream_identifier(self) -> global___ReadReq.Options.FilterOptions.Expression: ...
            @property
            def event_type(self) -> global___ReadReq.Options.FilterOptions.Expression: ...
            max: builtins.int
            @property
            def count(self) -> shared_pb2.Empty: ...
            checkpointIntervalMultiplier: builtins.int
            def __init__(
                self,
                *,
                stream_identifier: global___ReadReq.Options.FilterOptions.Expression | None = ...,
                event_type: global___ReadReq.Options.FilterOptions.Expression | None = ...,
                max: builtins.int = ...,
                count: shared_pb2.Empty | None = ...,
                checkpointIntervalMultiplier: builtins.int = ...,
            ) -> None: ...
            def HasField(self, field_name: typing_extensions.Literal["count", b"count", "event_type", b"event_type", "filter", b"filter", "max", b"max", "stream_identifier", b"stream_identifier", "window", b"window"]) -> builtins.bool: ...
            def ClearField(self, field_name: typing_extensions.Literal["checkpointIntervalMultiplier", b"checkpointIntervalMultiplier", "count", b"count", "event_type", b"event_type", "filter", b"filter", "max", b"max", "stream_identifier", b"stream_identifier", "window", b"window"]) -> None: ...
            @typing.overload
            def WhichOneof(self, oneof_group: typing_extensions.Literal["filter", b"filter"]) -> typing_extensions.Literal["stream_identifier", "event_type"] | None: ...
            @typing.overload
            def WhichOneof(self, oneof_group: typing_extensions.Literal["window", b"window"]) -> typing_extensions.Literal["max", "count"] | None: ...

        class UUIDOption(google.protobuf.message.Message):
            DESCRIPTOR: google.protobuf.descriptor.Descriptor

            STRUCTURED_FIELD_NUMBER: builtins.int
            STRING_FIELD_NUMBER: builtins.int
            @property
            def structured(self) -> shared_pb2.Empty: ...
            @property
            def string(self) -> shared_pb2.Empty: ...
            def __init__(
                self,
                *,
                structured: shared_pb2.Empty | None = ...,
                string: shared_pb2.Empty | None = ...,
            ) -> None: ...
            def HasField(self, field_name: typing_extensions.Literal["content", b"content", "string", b"string", "structured", b"structured"]) -> builtins.bool: ...
            def ClearField(self, field_name: typing_extensions.Literal["content", b"content", "string", b"string", "structured", b"structured"]) -> None: ...
            def WhichOneof(self, oneof_group: typing_extensions.Literal["content", b"content"]) -> typing_extensions.Literal["structured", "string"] | None: ...

        class ControlOption(google.protobuf.message.Message):
            DESCRIPTOR: google.protobuf.descriptor.Descriptor

            COMPATIBILITY_FIELD_NUMBER: builtins.int
            compatibility: builtins.int
            def __init__(
                self,
                *,
                compatibility: builtins.int = ...,
            ) -> None: ...
            def ClearField(self, field_name: typing_extensions.Literal["compatibility", b"compatibility"]) -> None: ...

        STREAM_FIELD_NUMBER: builtins.int
        ALL_FIELD_NUMBER: builtins.int
        READ_DIRECTION_FIELD_NUMBER: builtins.int
        RESOLVE_LINKS_FIELD_NUMBER: builtins.int
        COUNT_FIELD_NUMBER: builtins.int
        SUBSCRIPTION_FIELD_NUMBER: builtins.int
        FILTER_FIELD_NUMBER: builtins.int
        NO_FILTER_FIELD_NUMBER: builtins.int
        UUID_OPTION_FIELD_NUMBER: builtins.int
        CONTROL_OPTION_FIELD_NUMBER: builtins.int
        @property
        def stream(self) -> global___ReadReq.Options.StreamOptions: ...
        @property
        def all(self) -> global___ReadReq.Options.AllOptions: ...
        read_direction: global___ReadReq.Options.ReadDirection.ValueType
        resolve_links: builtins.bool
        count: builtins.int
        @property
        def subscription(self) -> global___ReadReq.Options.SubscriptionOptions: ...
        @property
        def filter(self) -> global___ReadReq.Options.FilterOptions: ...
        @property
        def no_filter(self) -> shared_pb2.Empty: ...
        @property
        def uuid_option(self) -> global___ReadReq.Options.UUIDOption: ...
        @property
        def control_option(self) -> global___ReadReq.Options.ControlOption: ...
        def __init__(
            self,
            *,
            stream: global___ReadReq.Options.StreamOptions | None = ...,
            all: global___ReadReq.Options.AllOptions | None = ...,
            read_direction: global___ReadReq.Options.ReadDirection.ValueType = ...,
            resolve_links: builtins.bool = ...,
            count: builtins.int = ...,
            subscription: global___ReadReq.Options.SubscriptionOptions | None = ...,
            filter: global___ReadReq.Options.FilterOptions | None = ...,
            no_filter: shared_pb2.Empty | None = ...,
            uuid_option: global___ReadReq.Options.UUIDOption | None = ...,
            control_option: global___ReadReq.Options.ControlOption | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["all", b"all", "control_option", b"control_option", "count", b"count", "count_option", b"count_option", "filter", b"filter", "filter_option", b"filter_option", "no_filter", b"no_filter", "stream", b"stream", "stream_option", b"stream_option", "subscription", b"subscription", "uuid_option", b"uuid_option"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["all", b"all", "control_option", b"control_option", "count", b"count", "count_option", b"count_option", "filter", b"filter", "filter_option", b"filter_option", "no_filter", b"no_filter", "read_direction", b"read_direction", "resolve_links", b"resolve_links", "stream", b"stream", "stream_option", b"stream_option", "subscription", b"subscription", "uuid_option", b"uuid_option"]) -> None: ...
        @typing.overload
        def WhichOneof(self, oneof_group: typing_extensions.Literal["count_option", b"count_option"]) -> typing_extensions.Literal["count", "subscription"] | None: ...
        @typing.overload
        def WhichOneof(self, oneof_group: typing_extensions.Literal["filter_option", b"filter_option"]) -> typing_extensions.Literal["filter", "no_filter"] | None: ...
        @typing.overload
        def WhichOneof(self, oneof_group: typing_extensions.Literal["stream_option", b"stream_option"]) -> typing_extensions.Literal["stream", "all"] | None: ...

    OPTIONS_FIELD_NUMBER: builtins.int
    @property
    def options(self) -> global___ReadReq.Options: ...
    def __init__(
        self,
        *,
        options: global___ReadReq.Options | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["options", b"options"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["options", b"options"]) -> None: ...

global___ReadReq = ReadReq

class ReadResp(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class ReadEvent(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        class RecordedEvent(google.protobuf.message.Message):
            DESCRIPTOR: google.protobuf.descriptor.Descriptor

            class MetadataEntry(google.protobuf.message.Message):
                DESCRIPTOR: google.protobuf.descriptor.Descriptor

                KEY_FIELD_NUMBER: builtins.int
                VALUE_FIELD_NUMBER: builtins.int
                key: builtins.str
                value: builtins.str
                def __init__(
                    self,
                    *,
                    key: builtins.str = ...,
                    value: builtins.str = ...,
                ) -> None: ...
                def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

            ID_FIELD_NUMBER: builtins.int
            STREAM_IDENTIFIER_FIELD_NUMBER: builtins.int
            STREAM_REVISION_FIELD_NUMBER: builtins.int
            PREPARE_POSITION_FIELD_NUMBER: builtins.int
            COMMIT_POSITION_FIELD_NUMBER: builtins.int
            METADATA_FIELD_NUMBER: builtins.int
            CUSTOM_METADATA_FIELD_NUMBER: builtins.int
            DATA_FIELD_NUMBER: builtins.int
            @property
            def id(self) -> shared_pb2.UUID: ...
            @property
            def stream_identifier(self) -> shared_pb2.StreamIdentifier: ...
            stream_revision: builtins.int
            prepare_position: builtins.int
            commit_position: builtins.int
            @property
            def metadata(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]: ...
            custom_metadata: builtins.bytes
            data: builtins.bytes
            def __init__(
                self,
                *,
                id: shared_pb2.UUID | None = ...,
                stream_identifier: shared_pb2.StreamIdentifier | None = ...,
                stream_revision: builtins.int = ...,
                prepare_position: builtins.int = ...,
                commit_position: builtins.int = ...,
                metadata: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
                custom_metadata: builtins.bytes = ...,
                data: builtins.bytes = ...,
            ) -> None: ...
            def HasField(self, field_name: typing_extensions.Literal["id", b"id", "stream_identifier", b"stream_identifier"]) -> builtins.bool: ...
            def ClearField(self, field_name: typing_extensions.Literal["commit_position", b"commit_position", "custom_metadata", b"custom_metadata", "data", b"data", "id", b"id", "metadata", b"metadata", "prepare_position", b"prepare_position", "stream_identifier", b"stream_identifier", "stream_revision", b"stream_revision"]) -> None: ...

        EVENT_FIELD_NUMBER: builtins.int
        LINK_FIELD_NUMBER: builtins.int
        COMMIT_POSITION_FIELD_NUMBER: builtins.int
        NO_POSITION_FIELD_NUMBER: builtins.int
        @property
        def event(self) -> global___ReadResp.ReadEvent.RecordedEvent: ...
        @property
        def link(self) -> global___ReadResp.ReadEvent.RecordedEvent: ...
        commit_position: builtins.int
        @property
        def no_position(self) -> shared_pb2.Empty: ...
        def __init__(
            self,
            *,
            event: global___ReadResp.ReadEvent.RecordedEvent | None = ...,
            link: global___ReadResp.ReadEvent.RecordedEvent | None = ...,
            commit_position: builtins.int = ...,
            no_position: shared_pb2.Empty | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["commit_position", b"commit_position", "event", b"event", "link", b"link", "no_position", b"no_position", "position", b"position"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["commit_position", b"commit_position", "event", b"event", "link", b"link", "no_position", b"no_position", "position", b"position"]) -> None: ...
        def WhichOneof(self, oneof_group: typing_extensions.Literal["position", b"position"]) -> typing_extensions.Literal["commit_position", "no_position"] | None: ...

    class SubscriptionConfirmation(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        SUBSCRIPTION_ID_FIELD_NUMBER: builtins.int
        subscription_id: builtins.str
        def __init__(
            self,
            *,
            subscription_id: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["subscription_id", b"subscription_id"]) -> None: ...

    class Checkpoint(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        COMMIT_POSITION_FIELD_NUMBER: builtins.int
        PREPARE_POSITION_FIELD_NUMBER: builtins.int
        commit_position: builtins.int
        prepare_position: builtins.int
        def __init__(
            self,
            *,
            commit_position: builtins.int = ...,
            prepare_position: builtins.int = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["commit_position", b"commit_position", "prepare_position", b"prepare_position"]) -> None: ...

    class StreamNotFound(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        STREAM_IDENTIFIER_FIELD_NUMBER: builtins.int
        @property
        def stream_identifier(self) -> shared_pb2.StreamIdentifier: ...
        def __init__(
            self,
            *,
            stream_identifier: shared_pb2.StreamIdentifier | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["stream_identifier", b"stream_identifier"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["stream_identifier", b"stream_identifier"]) -> None: ...

    EVENT_FIELD_NUMBER: builtins.int
    CONFIRMATION_FIELD_NUMBER: builtins.int
    CHECKPOINT_FIELD_NUMBER: builtins.int
    STREAM_NOT_FOUND_FIELD_NUMBER: builtins.int
    FIRST_STREAM_POSITION_FIELD_NUMBER: builtins.int
    LAST_STREAM_POSITION_FIELD_NUMBER: builtins.int
    LAST_ALL_STREAM_POSITION_FIELD_NUMBER: builtins.int
    @property
    def event(self) -> global___ReadResp.ReadEvent: ...
    @property
    def confirmation(self) -> global___ReadResp.SubscriptionConfirmation: ...
    @property
    def checkpoint(self) -> global___ReadResp.Checkpoint: ...
    @property
    def stream_not_found(self) -> global___ReadResp.StreamNotFound: ...
    first_stream_position: builtins.int
    last_stream_position: builtins.int
    @property
    def last_all_stream_position(self) -> shared_pb2.AllStreamPosition: ...
    def __init__(
        self,
        *,
        event: global___ReadResp.ReadEvent | None = ...,
        confirmation: global___ReadResp.SubscriptionConfirmation | None = ...,
        checkpoint: global___ReadResp.Checkpoint | None = ...,
        stream_not_found: global___ReadResp.StreamNotFound | None = ...,
        first_stream_position: builtins.int = ...,
        last_stream_position: builtins.int = ...,
        last_all_stream_position: shared_pb2.AllStreamPosition | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["checkpoint", b"checkpoint", "confirmation", b"confirmation", "content", b"content", "event", b"event", "first_stream_position", b"first_stream_position", "last_all_stream_position", b"last_all_stream_position", "last_stream_position", b"last_stream_position", "stream_not_found", b"stream_not_found"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["checkpoint", b"checkpoint", "confirmation", b"confirmation", "content", b"content", "event", b"event", "first_stream_position", b"first_stream_position", "last_all_stream_position", b"last_all_stream_position", "last_stream_position", b"last_stream_position", "stream_not_found", b"stream_not_found"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["content", b"content"]) -> typing_extensions.Literal["event", "confirmation", "checkpoint", "stream_not_found", "first_stream_position", "last_stream_position", "last_all_stream_position"] | None: ...

global___ReadResp = ReadResp

class AppendReq(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class Options(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        STREAM_IDENTIFIER_FIELD_NUMBER: builtins.int
        REVISION_FIELD_NUMBER: builtins.int
        NO_STREAM_FIELD_NUMBER: builtins.int
        ANY_FIELD_NUMBER: builtins.int
        STREAM_EXISTS_FIELD_NUMBER: builtins.int
        @property
        def stream_identifier(self) -> shared_pb2.StreamIdentifier: ...
        revision: builtins.int
        @property
        def no_stream(self) -> shared_pb2.Empty: ...
        @property
        def any(self) -> shared_pb2.Empty: ...
        @property
        def stream_exists(self) -> shared_pb2.Empty: ...
        def __init__(
            self,
            *,
            stream_identifier: shared_pb2.StreamIdentifier | None = ...,
            revision: builtins.int = ...,
            no_stream: shared_pb2.Empty | None = ...,
            any: shared_pb2.Empty | None = ...,
            stream_exists: shared_pb2.Empty | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["any", b"any", "expected_stream_revision", b"expected_stream_revision", "no_stream", b"no_stream", "revision", b"revision", "stream_exists", b"stream_exists", "stream_identifier", b"stream_identifier"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["any", b"any", "expected_stream_revision", b"expected_stream_revision", "no_stream", b"no_stream", "revision", b"revision", "stream_exists", b"stream_exists", "stream_identifier", b"stream_identifier"]) -> None: ...
        def WhichOneof(self, oneof_group: typing_extensions.Literal["expected_stream_revision", b"expected_stream_revision"]) -> typing_extensions.Literal["revision", "no_stream", "any", "stream_exists"] | None: ...

    class ProposedMessage(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        class MetadataEntry(google.protobuf.message.Message):
            DESCRIPTOR: google.protobuf.descriptor.Descriptor

            KEY_FIELD_NUMBER: builtins.int
            VALUE_FIELD_NUMBER: builtins.int
            key: builtins.str
            value: builtins.str
            def __init__(
                self,
                *,
                key: builtins.str = ...,
                value: builtins.str = ...,
            ) -> None: ...
            def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

        ID_FIELD_NUMBER: builtins.int
        METADATA_FIELD_NUMBER: builtins.int
        CUSTOM_METADATA_FIELD_NUMBER: builtins.int
        DATA_FIELD_NUMBER: builtins.int
        @property
        def id(self) -> shared_pb2.UUID: ...
        @property
        def metadata(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]: ...
        custom_metadata: builtins.bytes
        data: builtins.bytes
        def __init__(
            self,
            *,
            id: shared_pb2.UUID | None = ...,
            metadata: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
            custom_metadata: builtins.bytes = ...,
            data: builtins.bytes = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["id", b"id"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["custom_metadata", b"custom_metadata", "data", b"data", "id", b"id", "metadata", b"metadata"]) -> None: ...

    OPTIONS_FIELD_NUMBER: builtins.int
    PROPOSED_MESSAGE_FIELD_NUMBER: builtins.int
    @property
    def options(self) -> global___AppendReq.Options: ...
    @property
    def proposed_message(self) -> global___AppendReq.ProposedMessage: ...
    def __init__(
        self,
        *,
        options: global___AppendReq.Options | None = ...,
        proposed_message: global___AppendReq.ProposedMessage | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["content", b"content", "options", b"options", "proposed_message", b"proposed_message"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["content", b"content", "options", b"options", "proposed_message", b"proposed_message"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["content", b"content"]) -> typing_extensions.Literal["options", "proposed_message"] | None: ...

global___AppendReq = AppendReq

class AppendResp(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class Position(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        COMMIT_POSITION_FIELD_NUMBER: builtins.int
        PREPARE_POSITION_FIELD_NUMBER: builtins.int
        commit_position: builtins.int
        prepare_position: builtins.int
        def __init__(
            self,
            *,
            commit_position: builtins.int = ...,
            prepare_position: builtins.int = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["commit_position", b"commit_position", "prepare_position", b"prepare_position"]) -> None: ...

    class Success(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        CURRENT_REVISION_FIELD_NUMBER: builtins.int
        NO_STREAM_FIELD_NUMBER: builtins.int
        POSITION_FIELD_NUMBER: builtins.int
        NO_POSITION_FIELD_NUMBER: builtins.int
        current_revision: builtins.int
        @property
        def no_stream(self) -> shared_pb2.Empty: ...
        @property
        def position(self) -> global___AppendResp.Position: ...
        @property
        def no_position(self) -> shared_pb2.Empty: ...
        def __init__(
            self,
            *,
            current_revision: builtins.int = ...,
            no_stream: shared_pb2.Empty | None = ...,
            position: global___AppendResp.Position | None = ...,
            no_position: shared_pb2.Empty | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["current_revision", b"current_revision", "current_revision_option", b"current_revision_option", "no_position", b"no_position", "no_stream", b"no_stream", "position", b"position", "position_option", b"position_option"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["current_revision", b"current_revision", "current_revision_option", b"current_revision_option", "no_position", b"no_position", "no_stream", b"no_stream", "position", b"position", "position_option", b"position_option"]) -> None: ...
        @typing.overload
        def WhichOneof(self, oneof_group: typing_extensions.Literal["current_revision_option", b"current_revision_option"]) -> typing_extensions.Literal["current_revision", "no_stream"] | None: ...
        @typing.overload
        def WhichOneof(self, oneof_group: typing_extensions.Literal["position_option", b"position_option"]) -> typing_extensions.Literal["position", "no_position"] | None: ...

    class WrongExpectedVersion(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        CURRENT_REVISION_20_6_0_FIELD_NUMBER: builtins.int
        NO_STREAM_20_6_0_FIELD_NUMBER: builtins.int
        EXPECTED_REVISION_20_6_0_FIELD_NUMBER: builtins.int
        ANY_20_6_0_FIELD_NUMBER: builtins.int
        STREAM_EXISTS_20_6_0_FIELD_NUMBER: builtins.int
        CURRENT_REVISION_FIELD_NUMBER: builtins.int
        CURRENT_NO_STREAM_FIELD_NUMBER: builtins.int
        EXPECTED_REVISION_FIELD_NUMBER: builtins.int
        EXPECTED_ANY_FIELD_NUMBER: builtins.int
        EXPECTED_STREAM_EXISTS_FIELD_NUMBER: builtins.int
        EXPECTED_NO_STREAM_FIELD_NUMBER: builtins.int
        current_revision_20_6_0: builtins.int
        @property
        def no_stream_20_6_0(self) -> shared_pb2.Empty: ...
        expected_revision_20_6_0: builtins.int
        @property
        def any_20_6_0(self) -> shared_pb2.Empty: ...
        @property
        def stream_exists_20_6_0(self) -> shared_pb2.Empty: ...
        current_revision: builtins.int
        @property
        def current_no_stream(self) -> shared_pb2.Empty: ...
        expected_revision: builtins.int
        @property
        def expected_any(self) -> shared_pb2.Empty: ...
        @property
        def expected_stream_exists(self) -> shared_pb2.Empty: ...
        @property
        def expected_no_stream(self) -> shared_pb2.Empty: ...
        def __init__(
            self,
            *,
            current_revision_20_6_0: builtins.int = ...,
            no_stream_20_6_0: shared_pb2.Empty | None = ...,
            expected_revision_20_6_0: builtins.int = ...,
            any_20_6_0: shared_pb2.Empty | None = ...,
            stream_exists_20_6_0: shared_pb2.Empty | None = ...,
            current_revision: builtins.int = ...,
            current_no_stream: shared_pb2.Empty | None = ...,
            expected_revision: builtins.int = ...,
            expected_any: shared_pb2.Empty | None = ...,
            expected_stream_exists: shared_pb2.Empty | None = ...,
            expected_no_stream: shared_pb2.Empty | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["any_20_6_0", b"any_20_6_0", "current_no_stream", b"current_no_stream", "current_revision", b"current_revision", "current_revision_20_6_0", b"current_revision_20_6_0", "current_revision_option", b"current_revision_option", "current_revision_option_20_6_0", b"current_revision_option_20_6_0", "expected_any", b"expected_any", "expected_no_stream", b"expected_no_stream", "expected_revision", b"expected_revision", "expected_revision_20_6_0", b"expected_revision_20_6_0", "expected_revision_option", b"expected_revision_option", "expected_revision_option_20_6_0", b"expected_revision_option_20_6_0", "expected_stream_exists", b"expected_stream_exists", "no_stream_20_6_0", b"no_stream_20_6_0", "stream_exists_20_6_0", b"stream_exists_20_6_0"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["any_20_6_0", b"any_20_6_0", "current_no_stream", b"current_no_stream", "current_revision", b"current_revision", "current_revision_20_6_0", b"current_revision_20_6_0", "current_revision_option", b"current_revision_option", "current_revision_option_20_6_0", b"current_revision_option_20_6_0", "expected_any", b"expected_any", "expected_no_stream", b"expected_no_stream", "expected_revision", b"expected_revision", "expected_revision_20_6_0", b"expected_revision_20_6_0", "expected_revision_option", b"expected_revision_option", "expected_revision_option_20_6_0", b"expected_revision_option_20_6_0", "expected_stream_exists", b"expected_stream_exists", "no_stream_20_6_0", b"no_stream_20_6_0", "stream_exists_20_6_0", b"stream_exists_20_6_0"]) -> None: ...
        @typing.overload
        def WhichOneof(self, oneof_group: typing_extensions.Literal["current_revision_option", b"current_revision_option"]) -> typing_extensions.Literal["current_revision", "current_no_stream"] | None: ...
        @typing.overload
        def WhichOneof(self, oneof_group: typing_extensions.Literal["current_revision_option_20_6_0", b"current_revision_option_20_6_0"]) -> typing_extensions.Literal["current_revision_20_6_0", "no_stream_20_6_0"] | None: ...
        @typing.overload
        def WhichOneof(self, oneof_group: typing_extensions.Literal["expected_revision_option", b"expected_revision_option"]) -> typing_extensions.Literal["expected_revision", "expected_any", "expected_stream_exists", "expected_no_stream"] | None: ...
        @typing.overload
        def WhichOneof(self, oneof_group: typing_extensions.Literal["expected_revision_option_20_6_0", b"expected_revision_option_20_6_0"]) -> typing_extensions.Literal["expected_revision_20_6_0", "any_20_6_0", "stream_exists_20_6_0"] | None: ...

    SUCCESS_FIELD_NUMBER: builtins.int
    WRONG_EXPECTED_VERSION_FIELD_NUMBER: builtins.int
    @property
    def success(self) -> global___AppendResp.Success: ...
    @property
    def wrong_expected_version(self) -> global___AppendResp.WrongExpectedVersion: ...
    def __init__(
        self,
        *,
        success: global___AppendResp.Success | None = ...,
        wrong_expected_version: global___AppendResp.WrongExpectedVersion | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["result", b"result", "success", b"success", "wrong_expected_version", b"wrong_expected_version"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["result", b"result", "success", b"success", "wrong_expected_version", b"wrong_expected_version"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["result", b"result"]) -> typing_extensions.Literal["success", "wrong_expected_version"] | None: ...

global___AppendResp = AppendResp

class BatchAppendReq(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class Options(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        STREAM_IDENTIFIER_FIELD_NUMBER: builtins.int
        STREAM_POSITION_FIELD_NUMBER: builtins.int
        NO_STREAM_FIELD_NUMBER: builtins.int
        ANY_FIELD_NUMBER: builtins.int
        STREAM_EXISTS_FIELD_NUMBER: builtins.int
        DEADLINE_21_10_0_FIELD_NUMBER: builtins.int
        DEADLINE_FIELD_NUMBER: builtins.int
        @property
        def stream_identifier(self) -> shared_pb2.StreamIdentifier: ...
        stream_position: builtins.int
        @property
        def no_stream(self) -> google.protobuf.empty_pb2.Empty: ...
        @property
        def any(self) -> google.protobuf.empty_pb2.Empty: ...
        @property
        def stream_exists(self) -> google.protobuf.empty_pb2.Empty: ...
        @property
        def deadline_21_10_0(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
        @property
        def deadline(self) -> google.protobuf.duration_pb2.Duration: ...
        def __init__(
            self,
            *,
            stream_identifier: shared_pb2.StreamIdentifier | None = ...,
            stream_position: builtins.int = ...,
            no_stream: google.protobuf.empty_pb2.Empty | None = ...,
            any: google.protobuf.empty_pb2.Empty | None = ...,
            stream_exists: google.protobuf.empty_pb2.Empty | None = ...,
            deadline_21_10_0: google.protobuf.timestamp_pb2.Timestamp | None = ...,
            deadline: google.protobuf.duration_pb2.Duration | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["any", b"any", "deadline", b"deadline", "deadline_21_10_0", b"deadline_21_10_0", "deadline_option", b"deadline_option", "expected_stream_position", b"expected_stream_position", "no_stream", b"no_stream", "stream_exists", b"stream_exists", "stream_identifier", b"stream_identifier", "stream_position", b"stream_position"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["any", b"any", "deadline", b"deadline", "deadline_21_10_0", b"deadline_21_10_0", "deadline_option", b"deadline_option", "expected_stream_position", b"expected_stream_position", "no_stream", b"no_stream", "stream_exists", b"stream_exists", "stream_identifier", b"stream_identifier", "stream_position", b"stream_position"]) -> None: ...
        @typing.overload
        def WhichOneof(self, oneof_group: typing_extensions.Literal["deadline_option", b"deadline_option"]) -> typing_extensions.Literal["deadline_21_10_0", "deadline"] | None: ...
        @typing.overload
        def WhichOneof(self, oneof_group: typing_extensions.Literal["expected_stream_position", b"expected_stream_position"]) -> typing_extensions.Literal["stream_position", "no_stream", "any", "stream_exists"] | None: ...

    class ProposedMessage(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        class MetadataEntry(google.protobuf.message.Message):
            DESCRIPTOR: google.protobuf.descriptor.Descriptor

            KEY_FIELD_NUMBER: builtins.int
            VALUE_FIELD_NUMBER: builtins.int
            key: builtins.str
            value: builtins.str
            def __init__(
                self,
                *,
                key: builtins.str = ...,
                value: builtins.str = ...,
            ) -> None: ...
            def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

        ID_FIELD_NUMBER: builtins.int
        METADATA_FIELD_NUMBER: builtins.int
        CUSTOM_METADATA_FIELD_NUMBER: builtins.int
        DATA_FIELD_NUMBER: builtins.int
        @property
        def id(self) -> shared_pb2.UUID: ...
        @property
        def metadata(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]: ...
        custom_metadata: builtins.bytes
        data: builtins.bytes
        def __init__(
            self,
            *,
            id: shared_pb2.UUID | None = ...,
            metadata: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
            custom_metadata: builtins.bytes = ...,
            data: builtins.bytes = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["id", b"id"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["custom_metadata", b"custom_metadata", "data", b"data", "id", b"id", "metadata", b"metadata"]) -> None: ...

    CORRELATION_ID_FIELD_NUMBER: builtins.int
    OPTIONS_FIELD_NUMBER: builtins.int
    PROPOSED_MESSAGES_FIELD_NUMBER: builtins.int
    IS_FINAL_FIELD_NUMBER: builtins.int
    @property
    def correlation_id(self) -> shared_pb2.UUID: ...
    @property
    def options(self) -> global___BatchAppendReq.Options: ...
    @property
    def proposed_messages(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___BatchAppendReq.ProposedMessage]: ...
    is_final: builtins.bool
    def __init__(
        self,
        *,
        correlation_id: shared_pb2.UUID | None = ...,
        options: global___BatchAppendReq.Options | None = ...,
        proposed_messages: collections.abc.Iterable[global___BatchAppendReq.ProposedMessage] | None = ...,
        is_final: builtins.bool = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["correlation_id", b"correlation_id", "options", b"options"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["correlation_id", b"correlation_id", "is_final", b"is_final", "options", b"options", "proposed_messages", b"proposed_messages"]) -> None: ...

global___BatchAppendReq = BatchAppendReq

class BatchAppendResp(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class Success(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        CURRENT_REVISION_FIELD_NUMBER: builtins.int
        NO_STREAM_FIELD_NUMBER: builtins.int
        POSITION_FIELD_NUMBER: builtins.int
        NO_POSITION_FIELD_NUMBER: builtins.int
        current_revision: builtins.int
        @property
        def no_stream(self) -> google.protobuf.empty_pb2.Empty: ...
        @property
        def position(self) -> shared_pb2.AllStreamPosition: ...
        @property
        def no_position(self) -> google.protobuf.empty_pb2.Empty: ...
        def __init__(
            self,
            *,
            current_revision: builtins.int = ...,
            no_stream: google.protobuf.empty_pb2.Empty | None = ...,
            position: shared_pb2.AllStreamPosition | None = ...,
            no_position: google.protobuf.empty_pb2.Empty | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["current_revision", b"current_revision", "current_revision_option", b"current_revision_option", "no_position", b"no_position", "no_stream", b"no_stream", "position", b"position", "position_option", b"position_option"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["current_revision", b"current_revision", "current_revision_option", b"current_revision_option", "no_position", b"no_position", "no_stream", b"no_stream", "position", b"position", "position_option", b"position_option"]) -> None: ...
        @typing.overload
        def WhichOneof(self, oneof_group: typing_extensions.Literal["current_revision_option", b"current_revision_option"]) -> typing_extensions.Literal["current_revision", "no_stream"] | None: ...
        @typing.overload
        def WhichOneof(self, oneof_group: typing_extensions.Literal["position_option", b"position_option"]) -> typing_extensions.Literal["position", "no_position"] | None: ...

    CORRELATION_ID_FIELD_NUMBER: builtins.int
    ERROR_FIELD_NUMBER: builtins.int
    SUCCESS_FIELD_NUMBER: builtins.int
    STREAM_IDENTIFIER_FIELD_NUMBER: builtins.int
    STREAM_POSITION_FIELD_NUMBER: builtins.int
    NO_STREAM_FIELD_NUMBER: builtins.int
    ANY_FIELD_NUMBER: builtins.int
    STREAM_EXISTS_FIELD_NUMBER: builtins.int
    @property
    def correlation_id(self) -> shared_pb2.UUID: ...
    @property
    def error(self) -> status_pb2.Status: ...
    @property
    def success(self) -> global___BatchAppendResp.Success: ...
    @property
    def stream_identifier(self) -> shared_pb2.StreamIdentifier: ...
    stream_position: builtins.int
    @property
    def no_stream(self) -> google.protobuf.empty_pb2.Empty: ...
    @property
    def any(self) -> google.protobuf.empty_pb2.Empty: ...
    @property
    def stream_exists(self) -> google.protobuf.empty_pb2.Empty: ...
    def __init__(
        self,
        *,
        correlation_id: shared_pb2.UUID | None = ...,
        error: status_pb2.Status | None = ...,
        success: global___BatchAppendResp.Success | None = ...,
        stream_identifier: shared_pb2.StreamIdentifier | None = ...,
        stream_position: builtins.int = ...,
        no_stream: google.protobuf.empty_pb2.Empty | None = ...,
        any: google.protobuf.empty_pb2.Empty | None = ...,
        stream_exists: google.protobuf.empty_pb2.Empty | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["any", b"any", "correlation_id", b"correlation_id", "error", b"error", "expected_stream_position", b"expected_stream_position", "no_stream", b"no_stream", "result", b"result", "stream_exists", b"stream_exists", "stream_identifier", b"stream_identifier", "stream_position", b"stream_position", "success", b"success"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["any", b"any", "correlation_id", b"correlation_id", "error", b"error", "expected_stream_position", b"expected_stream_position", "no_stream", b"no_stream", "result", b"result", "stream_exists", b"stream_exists", "stream_identifier", b"stream_identifier", "stream_position", b"stream_position", "success", b"success"]) -> None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal["expected_stream_position", b"expected_stream_position"]) -> typing_extensions.Literal["stream_position", "no_stream", "any", "stream_exists"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal["result", b"result"]) -> typing_extensions.Literal["error", "success"] | None: ...

global___BatchAppendResp = BatchAppendResp

class DeleteReq(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class Options(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        STREAM_IDENTIFIER_FIELD_NUMBER: builtins.int
        REVISION_FIELD_NUMBER: builtins.int
        NO_STREAM_FIELD_NUMBER: builtins.int
        ANY_FIELD_NUMBER: builtins.int
        STREAM_EXISTS_FIELD_NUMBER: builtins.int
        @property
        def stream_identifier(self) -> shared_pb2.StreamIdentifier: ...
        revision: builtins.int
        @property
        def no_stream(self) -> shared_pb2.Empty: ...
        @property
        def any(self) -> shared_pb2.Empty: ...
        @property
        def stream_exists(self) -> shared_pb2.Empty: ...
        def __init__(
            self,
            *,
            stream_identifier: shared_pb2.StreamIdentifier | None = ...,
            revision: builtins.int = ...,
            no_stream: shared_pb2.Empty | None = ...,
            any: shared_pb2.Empty | None = ...,
            stream_exists: shared_pb2.Empty | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["any", b"any", "expected_stream_revision", b"expected_stream_revision", "no_stream", b"no_stream", "revision", b"revision", "stream_exists", b"stream_exists", "stream_identifier", b"stream_identifier"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["any", b"any", "expected_stream_revision", b"expected_stream_revision", "no_stream", b"no_stream", "revision", b"revision", "stream_exists", b"stream_exists", "stream_identifier", b"stream_identifier"]) -> None: ...
        def WhichOneof(self, oneof_group: typing_extensions.Literal["expected_stream_revision", b"expected_stream_revision"]) -> typing_extensions.Literal["revision", "no_stream", "any", "stream_exists"] | None: ...

    OPTIONS_FIELD_NUMBER: builtins.int
    @property
    def options(self) -> global___DeleteReq.Options: ...
    def __init__(
        self,
        *,
        options: global___DeleteReq.Options | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["options", b"options"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["options", b"options"]) -> None: ...

global___DeleteReq = DeleteReq

class DeleteResp(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class Position(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        COMMIT_POSITION_FIELD_NUMBER: builtins.int
        PREPARE_POSITION_FIELD_NUMBER: builtins.int
        commit_position: builtins.int
        prepare_position: builtins.int
        def __init__(
            self,
            *,
            commit_position: builtins.int = ...,
            prepare_position: builtins.int = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["commit_position", b"commit_position", "prepare_position", b"prepare_position"]) -> None: ...

    POSITION_FIELD_NUMBER: builtins.int
    NO_POSITION_FIELD_NUMBER: builtins.int
    @property
    def position(self) -> global___DeleteResp.Position: ...
    @property
    def no_position(self) -> shared_pb2.Empty: ...
    def __init__(
        self,
        *,
        position: global___DeleteResp.Position | None = ...,
        no_position: shared_pb2.Empty | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["no_position", b"no_position", "position", b"position", "position_option", b"position_option"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["no_position", b"no_position", "position", b"position", "position_option", b"position_option"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["position_option", b"position_option"]) -> typing_extensions.Literal["position", "no_position"] | None: ...

global___DeleteResp = DeleteResp

class TombstoneReq(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class Options(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        STREAM_IDENTIFIER_FIELD_NUMBER: builtins.int
        REVISION_FIELD_NUMBER: builtins.int
        NO_STREAM_FIELD_NUMBER: builtins.int
        ANY_FIELD_NUMBER: builtins.int
        STREAM_EXISTS_FIELD_NUMBER: builtins.int
        @property
        def stream_identifier(self) -> shared_pb2.StreamIdentifier: ...
        revision: builtins.int
        @property
        def no_stream(self) -> shared_pb2.Empty: ...
        @property
        def any(self) -> shared_pb2.Empty: ...
        @property
        def stream_exists(self) -> shared_pb2.Empty: ...
        def __init__(
            self,
            *,
            stream_identifier: shared_pb2.StreamIdentifier | None = ...,
            revision: builtins.int = ...,
            no_stream: shared_pb2.Empty | None = ...,
            any: shared_pb2.Empty | None = ...,
            stream_exists: shared_pb2.Empty | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["any", b"any", "expected_stream_revision", b"expected_stream_revision", "no_stream", b"no_stream", "revision", b"revision", "stream_exists", b"stream_exists", "stream_identifier", b"stream_identifier"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["any", b"any", "expected_stream_revision", b"expected_stream_revision", "no_stream", b"no_stream", "revision", b"revision", "stream_exists", b"stream_exists", "stream_identifier", b"stream_identifier"]) -> None: ...
        def WhichOneof(self, oneof_group: typing_extensions.Literal["expected_stream_revision", b"expected_stream_revision"]) -> typing_extensions.Literal["revision", "no_stream", "any", "stream_exists"] | None: ...

    OPTIONS_FIELD_NUMBER: builtins.int
    @property
    def options(self) -> global___TombstoneReq.Options: ...
    def __init__(
        self,
        *,
        options: global___TombstoneReq.Options | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["options", b"options"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["options", b"options"]) -> None: ...

global___TombstoneReq = TombstoneReq

class TombstoneResp(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class Position(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        COMMIT_POSITION_FIELD_NUMBER: builtins.int
        PREPARE_POSITION_FIELD_NUMBER: builtins.int
        commit_position: builtins.int
        prepare_position: builtins.int
        def __init__(
            self,
            *,
            commit_position: builtins.int = ...,
            prepare_position: builtins.int = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["commit_position", b"commit_position", "prepare_position", b"prepare_position"]) -> None: ...

    POSITION_FIELD_NUMBER: builtins.int
    NO_POSITION_FIELD_NUMBER: builtins.int
    @property
    def position(self) -> global___TombstoneResp.Position: ...
    @property
    def no_position(self) -> shared_pb2.Empty: ...
    def __init__(
        self,
        *,
        position: global___TombstoneResp.Position | None = ...,
        no_position: shared_pb2.Empty | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["no_position", b"no_position", "position", b"position", "position_option", b"position_option"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["no_position", b"no_position", "position", b"position", "position_option", b"position_option"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["position_option", b"position_option"]) -> typing_extensions.Literal["position", "no_position"] | None: ...

global___TombstoneResp = TombstoneResp



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import cluster_pb2 as cluster__pb2
from . import shared_pb2 as shared__pb2


class GossipStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Update = channel.unary_unary(
                '/event_store.cluster.Gossip/Update',
                request_serializer=cluster__pb2.GossipRequest.SerializeToString,
                response_deserializer=cluster__pb2.ClusterInfo.FromString,
                )
        self.Read = channel.unary_unary(
                '/event_store.cluster.Gossip/Read',
                request_serializer=shared__pb2.Empty.SerializeToString,
                response_deserializer=cluster__pb2.ClusterInfo.FromString,
                )


class GossipServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Update(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Read(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_GossipServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Update': grpc.unary_unary_rpc_method_handler(
                    servicer.Update,
                    request_deserializer=cluster__pb2.GossipRequest.FromString,
                    response_serializer=cluster__pb2.ClusterInfo.SerializeToString,
            ),
            'Read': grpc.unary_unary_rpc_method_handler(
                    servicer.Read,
                    request_deserializer=shared__pb2.Empty.FromString,
                    response_serializer=cluster__pb2.ClusterInfo.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'event_store.cluster.Gossip', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class Gossip(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Update(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/event_store.cluster.Gossip/Update',
            cluster__pb2.GossipRequest.SerializeToString,
            cluster__pb2.ClusterInfo.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Read(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/event_store.cluster.Gossip/Read',
            shared__pb2.Empty.SerializeToString,
            cluster__pb2.ClusterInfo.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)


class ElectionsStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.ViewChange = channel.unary_unary(
                '/event_store.cluster.Elections/ViewChange',
                request_serializer=cluster__pb2.ViewChangeRequest.SerializeToString,
                response_deserializer=shared__pb2.Empty.FromString,
                )
        self.ViewChangeProof = channel.unary_unary(
                '/event_store.cluster.Elections/ViewChangeProof',
                request_serializer=cluster__pb2.ViewChangeProofRequest.SerializeToString,
                response_deserializer=shared__pb2.Empty.FromString,
                )
        self.Prepare = channel.unary_unary(
                '/event_store.cluster.Elections/Prepare',
                request_serializer=cluster__pb2.PrepareRequest.SerializeToString,
                response_deserializer=shared__pb2.Empty.FromString,
                )
        self.PrepareOk = channel.unary_unary(
                '/event_store.cluster.Elections/PrepareOk',
                request_serializer=cluster__pb2.PrepareOkRequest.SerializeToString,
                response_deserializer=shared__pb2.Empty.FromString,
                )
        self.Proposal = channel.unary_unary(
                '/event_store.cluster.Elections/Proposal',
                request_serializer=cluster__pb2.ProposalRequest.SerializeToString,
                response_deserializer=shared__pb2.Empty.FromString,
                )
        self.Accept = channel.unary_unary(
                '/event_store.cluster.Elections/Accept',
                request_serializer=cluster__pb2.AcceptRequest.SerializeToString,
                response_deserializer=shared__pb2.Empty.FromString,
                )
        self.LeaderIsResigning = channel.unary_unary(
                '/event_store.cluster.Elections/LeaderIsResigning',
                request_serializer=cluster__pb2.LeaderIsResigningRequest.SerializeToString,
                response_deserializer=shared__pb2.Empty.FromString,
                )
        self.LeaderIsResigningOk = channel.unary_unary(
                '/event_store.cluster.Elections/LeaderIsResigningOk',
                request_serializer=cluster__pb2.LeaderIsResigningOkRequest.SerializeToString,
                response_deserializer=shared__pb2.Empty.FromString,
                )


class ElectionsServicer(object):
    """Missing associated documentation comment in .proto file."""

    def ViewChange(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ViewChangeProof(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Prepare(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def PrepareOk(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Proposal(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Accept(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def LeaderIsResigning(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def LeaderIsResigningOk(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_ElectionsServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'ViewChange': grpc.unary_unary_rpc_method_handler(
                    servicer.ViewChange,
                    request_deserializer=cluster__pb2.ViewChangeRequest.FromString,
                    response_serializer=shared__pb2.Empty.SerializeToString,
            ),
            'ViewChangeProof': grpc.unary_unary_rpc_method_handler(
                    servicer.ViewChangeProof,
                    request_deserializer=cluster__pb2.ViewChangeProofRequest.FromString,
                    response_serializer=shared__pb2.Empty.SerializeToString,
            ),
            'Prepare': grpc.unary_unary_rpc_method_handler(
                    servicer.Prepare,
                    request_deserializer=cluster__pb2.PrepareRequest.FromString,
                    response_serializer=shared__pb2.Empty.SerializeToString,
            ),
            'PrepareOk': grpc.unary_unary_rpc_method_handler(
                    servicer.PrepareOk,
                    request_deserializer=cluster__pb2.PrepareOkRequest.FromString,
                    response_serializer=shared__pb2.Empty.SerializeToString,
            ),
            'Proposal': grpc.unary_unary_rpc_method_handler(
                    servicer.Proposal,
                    request_deserializer=cluster__pb2.ProposalRequest.FromString,
                    response_serializer=shared__pb2.Empty.SerializeToString,
            ),
            'Accept': grpc.unary_unary_rpc_method_handler(
                    servicer.Accept,
                    request_deserializer=cluster__pb2.AcceptRequest.FromString,
                    response_serializer=shared__pb2.Empty.SerializeToString,
            ),
            'LeaderIsResigning': grpc.unary_unary_rpc_method_handler(
                    servicer.LeaderIsResigning,
                    request_deserializer=cluster__pb2.LeaderIsResigningRequest.FromString,
                    response_serializer=shared__pb2.Empty.SerializeToString,
            ),
            'LeaderIsResigningOk': grpc.unary_unary_rpc_method_handler(
                    servicer.LeaderIsResigningOk,
                    request_deserializer=cluster__pb2.LeaderIsResigningOkRequest.FromString,
                    response_serializer=shared__pb2.Empty.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'event_store.cluster.Elections', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class Elections(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def ViewChange(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/event_store.cluster.Elections/ViewChange',
            cluster__pb2.ViewChangeRequest.SerializeToString,
            shared__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ViewChangeProof(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/event_store.cluster.Elections/ViewChangeProof',
            cluster__pb2.ViewChangeProofRequest.SerializeToString,
            shared__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Prepare(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/event_store.cluster.Elections/Prepare',
            cluster__pb2.PrepareRequest.SerializeToString,
            shared__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def PrepareOk(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/event_store.cluster.Elections/PrepareOk',
            cluster__pb2.PrepareOkRequest.SerializeToString,
            shared__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Proposal(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/event_store.cluster.Elections/Proposal',
            cluster__pb2.ProposalRequest.SerializeToString,
            shared__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Accept(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/event_store.cluster.Elections/Accept',
            cluster__pb2.AcceptRequest.SerializeToString,
            shared__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def LeaderIsResigning(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/event_store.cluster.Elections/LeaderIsResigning',
            cluster__pb2.LeaderIsResigningRequest.SerializeToString,
            shared__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def LeaderIsResigningOk(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/event_store.cluster.Elections/LeaderIsResigningOk',
            cluster__pb2.LeaderIsResigningOkRequest.SerializeToString,
            shared__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import shared_pb2
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class GossipRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    INFO_FIELD_NUMBER: builtins.int
    SERVER_FIELD_NUMBER: builtins.int
    @property
    def info(self) -> global___ClusterInfo: ...
    @property
    def server(self) -> global___EndPoint: ...
    def __init__(
        self,
        *,
        info: global___ClusterInfo | None = ...,
        server: global___EndPoint | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["info", b"info", "server", b"server"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["info", b"info", "server", b"server"]) -> None: ...

global___GossipRequest = GossipRequest

class ViewChangeRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SERVER_ID_FIELD_NUMBER: builtins.int
    SERVER_HTTP_FIELD_NUMBER: builtins.int
    ATTEMPTED_VIEW_FIELD_NUMBER: builtins.int
    @property
    def server_id(self) -> shared_pb2.UUID: ...
    @property
    def server_http(self) -> global___EndPoint: ...
    attempted_view: builtins.int
    def __init__(
        self,
        *,
        server_id: shared_pb2.UUID | None = ...,
        server_http: global___EndPoint | None = ...,
        attempted_view: builtins.int = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["server_http", b"server_http", "server_id", b"server_id"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["attempted_view", b"attempted_view", "server_http", b"server_http", "server_id", b"server_id"]) -> None: ...

global___ViewChangeRequest = ViewChangeRequest

class ViewChangeProofRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SERVER_ID_FIELD_NUMBER: builtins.int
    SERVER_HTTP_FIELD_NUMBER: builtins.int
    INSTALLED_VIEW_FIELD_NUMBER: builtins.int
    @property
    def server_id(self) -> shared_pb2.UUID: ...
    @property
    def server_http(self) -> global___EndPoint: ...
    installed_view: builtins.int
    def __init__(
        self,
        *,
        server_id: shared_pb2.UUID | None = ...,
        server_http: global___EndPoint | None = ...,
        installed_view: builtins.int = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["server_http", b"server_http", "server_id", b"server_id"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["installed_view", b"installed_view", "server_http", b"server_http", "server_id", b"server_id"]) -> None: ...

global___ViewChangeProofRequest = ViewChangeProofRequest

class PrepareRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SERVER_ID_FIELD_NUMBER: builtins.int
    SERVER_HTTP_FIELD_NUMBER: builtins.int
    VIEW_FIELD_NUMBER: builtins.int
    @property
    def server_id(self) -> shared_pb2.UUID: ...
    @property
    def server_http(self) -> global___EndPoint: ...
    view: builtins.int
    def __init__(
        self,
        *,
        server_id: shared_pb2.UUID | None = ...,
        server_http: global___EndPoint | None = ...,
        view: builtins.int = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["server_http", b"server_http", "server_id", b"server_id"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["server_http", b"server_http", "server_id", b"server_id", "view", b"view"]) -> None: ...

global___PrepareRequest = PrepareRequest

class PrepareOkRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VIEW_FIELD_NUMBER: builtins.int
    SERVER_ID_FIELD_NUMBER: builtins.int
    SERVER_HTTP_FIELD_NUMBER: builtins.int
    EPOCH_NUMBER_FIELD_NUMBER: builtins.int
    EPOCH_POSITION_FIELD_NUMBER: builtins.int
    EPOCH_ID_FIELD_NUMBER: builtins.int
    EPOCH_LEADER_INSTANCE_ID_FIELD_NUMBER: builtins.int
    LAST_COMMIT_POSITION_FIELD_NUMBER: builtins.int
    WRITER_CHECKPOINT_FIELD_NUMBER: builtins.int
    CHASER_CHECKPOINT_FIELD_NUMBER: builtins.int
    NODE_PRIORITY_FIELD_NUMBER: builtins.int
    CLUSTER_INFO_FIELD_NUMBER: builtins.int
    view: builtins.int
    @property
    def server_id(self) -> shared_pb2.UUID: ...
    @property
    def server_http(self) -> global___EndPoint: ...
    epoch_number: builtins.int
    epoch_position: builtins.int
    @property
    def epoch_id(self) -> shared_pb2.UUID: ...
    @property
    def epoch_leader_instance_id(self) -> shared_pb2.UUID: ...
    last_commit_position: builtins.int
    writer_checkpoint: builtins.int
    chaser_checkpoint: builtins.int
    node_priority: builtins.int
    @property
    def cluster_info(self) -> global___ClusterInfo: ...
    def __init__(
        self,
        *,
        view: builtins.int = ...,
        server_id: shared_pb2.UUID | None = ...,
        server_http: global___EndPoint | None = ...,
        epoch_number: builtins.int = ...,
        epoch_position: builtins.int = ...,
        epoch_id: shared_pb2.UUID | None = ...,
        epoch_leader_instance_id: shared_pb2.UUID | None = ...,
        last_commit_position: builtins.int = ...,
        writer_checkpoint: builtins.int = ...,
        chaser_checkpoint: builtins.int = ...,
        node_priority: builtins.int = ...,
        cluster_info: global___ClusterInfo | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["cluster_info", b"cluster_info", "epoch_id", b"epoch_id", "epoch_leader_instance_id", b"epoch_leader_instance_id", "server_http", b"server_http", "server_id", b"server_id"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["chaser_checkpoint", b"chaser_checkpoint", "cluster_info", b"cluster_info", "epoch_id", b"epoch_id", "epoch_leader_instance_id", b"epoch_leader_instance_id", "epoch_number", b"epoch_number", "epoch_position", b"epoch_position", "last_commit_position", b"last_commit_position", "node_priority", b"node_priority", "server_http", b"server_http", "server_id", b"server_id", "view", b"view", "writer_checkpoint", b"writer_checkpoint"]) -> None: ...

global___PrepareOkRequest = PrepareOkRequest

class ProposalRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SERVER_ID_FIELD_NUMBER: builtins.int
    SERVER_HTTP_FIELD_NUMBER: builtins.int
    LEADER_ID_FIELD_NUMBER: builtins.int
    LEADER_HTTP_FIELD_NUMBER: builtins.int
    VIEW_FIELD_NUMBER: builtins.int
    EPOCH_NUMBER_FIELD_NUMBER: builtins.int
    EPOCH_POSITION_FIELD_NUMBER: builtins.int
    EPOCH_ID_FIELD_NUMBER: builtins.int
    EPOCH_LEADER_INSTANCE_ID_FIELD_NUMBER: builtins.int
    LAST_COMMIT_POSITION_FIELD_NUMBER: builtins.int
    WRITER_CHECKPOINT_FIELD_NUMBER: builtins.int
    CHASER_CHECKPOINT_FIELD_NUMBER: builtins.int
    NODE_PRIORITY_FIELD_NUMBER: builtins.int
    @property
    def server_id(self) -> shared_pb2.UUID: ...
    @property
    def server_http(self) -> global___EndPoint: ...
    @property
    def leader_id(self) -> shared_pb2.UUID: ...
    @property
    def leader_http(self) -> global___EndPoint: ...
    view: builtins.int
    epoch_number: builtins.int
    epoch_position: builtins.int
    @property
    def epoch_id(self) -> shared_pb2.UUID: ...
    @property
    def epoch_leader_instance_id(self) -> shared_pb2.UUID: ...
    last_commit_position: builtins.int
    writer_checkpoint: builtins.int
    chaser_checkpoint: builtins.int
    node_priority: builtins.int
    def __init__(
        self,
        *,
        server_id: shared_pb2.UUID | None = ...,
        server_http: global___EndPoint | None = ...,
        leader_id: shared_pb2.UUID | None = ...,
        leader_http: global___EndPoint | None = ...,
        view: builtins.int = ...,
        epoch_number: builtins.int = ...,
        epoch_position: builtins.int = ...,
        epoch_id: shared_pb2.UUID | None = ...,
        epoch_leader_instance_id: shared_pb2.UUID | None = ...,
        last_commit_position: builtins.int = ...,
        writer_checkpoint: builtins.int = ...,
        chaser_checkpoint: builtins.int = ...,
        node_priority: builtins.int = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["epoch_id", b"epoch_id", "epoch_leader_instance_id", b"epoch_leader_instance_id", "leader_http", b"leader_http", "leader_id", b"leader_id", "server_http", b"server_http", "server_id", b"server_id"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["chaser_checkpoint", b"chaser_checkpoint", "epoch_id", b"epoch_id", "epoch_leader_instance_id", b"epoch_leader_instance_id", "epoch_number", b"epoch_number", "epoch_position", b"epoch_position", "last_commit_position", b"last_commit_position", "leader_http", b"leader_http", "leader_id", b"leader_id", "node_priority", b"node_priority", "server_http", b"server_http", "server_id", b"server_id", "view", b"view", "writer_checkpoint", b"writer_checkpoint"]) -> None: ...

global___ProposalRequest = ProposalRequest

class AcceptRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SERVER_ID_FIELD_NUMBER: builtins.int
    SERVER_HTTP_FIELD_NUMBER: builtins.int
    LEADER_ID_FIELD_NUMBER: builtins.int
    LEADER_HTTP_FIELD_NUMBER: builtins.int
    VIEW_FIELD_NUMBER: builtins.int
    @property
    def server_id(self) -> shared_pb2.UUID: ...
    @property
    def server_http(self) -> global___EndPoint: ...
    @property
    def leader_id(self) -> shared_pb2.UUID: ...
    @property
    def leader_http(self) -> global___EndPoint: ...
    view: builtins.int
    def __init__(
        self,
        *,
        server_id: shared_pb2.UUID | None = ...,
        server_http: global___EndPoint | None = ...,
        leader_id: shared_pb2.UUID | None = ...,
        leader_http: global___EndPoint | None = ...,
        view: builtins.int = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["leader_http", b"leader_http", "leader_id", b"leader_id", "server_http", b"server_http", "server_id", b"server_id"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["leader_http", b"leader_http", "leader_id", b"leader_id", "server_http", b"server_http", "server_id", b"server_id", "view", b"view"]) -> None: ...

global___AcceptRequest = AcceptRequest

class LeaderIsResigningRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LEADER_ID_FIELD_NUMBER: builtins.int
    LEADER_HTTP_FIELD_NUMBER: builtins.int
    @property
    def leader_id(self) -> shared_pb2.UUID: ...
    @property
    def leader_http(self) -> global___EndPoint: ...
    def __init__(
        self,
        *,
        leader_id: shared_pb2.UUID | None = ...,
        leader_http: global___EndPoint | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["leader_http", b"leader_http", "leader_id", b"leader_id"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["leader_http", b"leader_http", "leader_id", b"leader_id"]) -> None: ...

global___LeaderIsResigningRequest = LeaderIsResigningRequest

class LeaderIsResigningOkRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LEADER_ID_FIELD_NUMBER: builtins.int
    LEADER_HTTP_FIELD_NUMBER: builtins.int
    SERVER_ID_FIELD_NUMBER: builtins.int
    SERVER_HTTP_FIELD_NUMBER: builtins.int
    @property
    def leader_id(self) -> shared_pb2.UUID: ...
    @property
    def leader_http(self) -> global___EndPoint: ...
    @property
    def server_id(self) -> shared_pb2.UUID: ...
    @property
    def server_http(self) -> global___EndPoint: ...
    def __init__(
        self,
        *,
        leader_id: shared_pb2.UUID | None = ...,
        leader_http: global___EndPoint | None = ...,
        server_id: shared_pb2.UUID | None = ...,
        server_http: global___EndPoint | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["leader_http", b"leader_http", "leader_id", b"leader_id", "server_http", b"server_http", "server_id", b"server_id"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["leader_http", b"leader_http", "leader_id", b"leader_id", "server_http", b"server_http", "server_id", b"server_id"]) -> None: ...

global___LeaderIsResigningOkRequest = LeaderIsResigningOkRequest

class ClusterInfo(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MEMBERS_FIELD_NUMBER: builtins.int
    @property
    def members(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___MemberInfo]: ...
    def __init__(
        self,
        *,
        members: collections.abc.Iterable[global___MemberInfo] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["members", b"members"]) -> None: ...

global___ClusterInfo = ClusterInfo

class EndPoint(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ADDRESS_FIELD_NUMBER: builtins.int
    PORT_FIELD_NUMBER: builtins.int
    address: builtins.str
    port: builtins.int
    def __init__(
        self,
        *,
        address: builtins.str = ...,
        port: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["address", b"address", "port", b"port"]) -> None: ...

global___EndPoint = EndPoint

class MemberInfo(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _VNodeState:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _VNodeStateEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[MemberInfo._VNodeState.ValueType], builtins.type):  # noqa: F821
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        Initializing: MemberInfo._VNodeState.ValueType  # 0
        DiscoverLeader: MemberInfo._VNodeState.ValueType  # 1
        Unknown: MemberInfo._VNodeState.ValueType  # 2
        PreReplica: MemberInfo._VNodeState.ValueType  # 3
        CatchingUp: MemberInfo._VNodeState.ValueType  # 4
        Clone: MemberInfo._VNodeState.ValueType  # 5
        Follower: MemberInfo._VNodeState.ValueType  # 6
        PreLeader: MemberInfo._VNodeState.ValueType  # 7
        Leader: MemberInfo._VNodeState.ValueType  # 8
        Manager: MemberInfo._VNodeState.ValueType  # 9
        ShuttingDown: MemberInfo._VNodeState.ValueType  # 10
        Shutdown: MemberInfo._VNodeState.ValueType  # 11
        ReadOnlyLeaderless: MemberInfo._VNodeState.ValueType  # 12
        PreReadOnlyReplica: MemberInfo._VNodeState.ValueType  # 13
        ReadOnlyReplica: MemberInfo._VNodeState.ValueType  # 14
        ResigningLeader: MemberInfo._VNodeState.ValueType  # 15

    class VNodeState(_VNodeState, metaclass=_VNodeStateEnumTypeWrapper): ...
    Initializing: MemberInfo.VNodeState.ValueType  # 0
    DiscoverLeader: MemberInfo.VNodeState.ValueType  # 1
    Unknown: MemberInfo.VNodeState.ValueType  # 2
    PreReplica: MemberInfo.VNodeState.ValueType  # 3
    CatchingUp: MemberInfo.VNodeState.ValueType  # 4
    Clone: MemberInfo.VNodeState.ValueType  # 5
    Follower: MemberInfo.VNodeState.ValueType  # 6
    PreLeader: MemberInfo.VNodeState.ValueType  # 7
    Leader: MemberInfo.VNodeState.ValueType  # 8
    Manager: MemberInfo.VNodeState.ValueType  # 9
    ShuttingDown: MemberInfo.VNodeState.ValueType  # 10
    Shutdown: MemberInfo.VNodeState.ValueType  # 11
    ReadOnlyLeaderless: MemberInfo.VNodeState.ValueType  # 12
    PreReadOnlyReplica: MemberInfo.VNodeState.ValueType  # 13
    ReadOnlyReplica: MemberInfo.VNodeState.ValueType  # 14
    ResigningLeader: MemberInfo.VNodeState.ValueType  # 15

    INSTANCE_ID_FIELD_NUMBER: builtins.int
    TIME_STAMP_FIELD_NUMBER: builtins.int
    STATE_FIELD_NUMBER: builtins.int
    IS_ALIVE_FIELD_NUMBER: builtins.int
    HTTP_END_POINT_FIELD_NUMBER: builtins.int
    INTERNAL_TCP_FIELD_NUMBER: builtins.int
    EXTERNAL_TCP_FIELD_NUMBER: builtins.int
    INTERNAL_TCP_USES_TLS_FIELD_NUMBER: builtins.int
    EXTERNAL_TCP_USES_TLS_FIELD_NUMBER: builtins.int
    LAST_COMMIT_POSITION_FIELD_NUMBER: builtins.int
    WRITER_CHECKPOINT_FIELD_NUMBER: builtins.int
    CHASER_CHECKPOINT_FIELD_NUMBER: builtins.int
    EPOCH_POSITION_FIELD_NUMBER: builtins.int
    EPOCH_NUMBER_FIELD_NUMBER: builtins.int
    EPOCH_ID_FIELD_NUMBER: builtins.int
    NODE_PRIORITY_FIELD_NUMBER: builtins.int
    IS_READ_ONLY_REPLICA_FIELD_NUMBER: builtins.int
    ADVERTISE_HOST_TO_CLIENT_AS_FIELD_NUMBER: builtins.int
    ADVERTISE_HTTP_PORT_TO_CLIENT_AS_FIELD_NUMBER: builtins.int
    ADVERTISE_TCP_PORT_TO_CLIENT_AS_FIELD_NUMBER: builtins.int
    @property
    def instance_id(self) -> shared_pb2.UUID: ...
    time_stamp: builtins.int
    state: global___MemberInfo.VNodeState.ValueType
    is_alive: builtins.bool
    @property
    def http_end_point(self) -> global___EndPoint: ...
    @property
    def internal_tcp(self) -> global___EndPoint: ...
    @property
    def external_tcp(self) -> global___EndPoint: ...
    internal_tcp_uses_tls: builtins.bool
    external_tcp_uses_tls: builtins.bool
    last_commit_position: builtins.int
    writer_checkpoint: builtins.int
    chaser_checkpoint: builtins.int
    epoch_position: builtins.int
    epoch_number: builtins.int
    @property
    def epoch_id(self) -> shared_pb2.UUID: ...
    node_priority: builtins.int
    is_read_only_replica: builtins.bool
    advertise_host_to_client_as: builtins.str
    advertise_http_port_to_client_as: builtins.int
    advertise_tcp_port_to_client_as: builtins.int
    def __init__(
        self,
        *,
        instance_id: shared_pb2.UUID | None = ...,
        time_stamp: builtins.int = ...,
        state: global___MemberInfo.VNodeState.ValueType = ...,
        is_alive: builtins.bool = ...,
        http_end_point: global___EndPoint | None = ...,
        internal_tcp: global___EndPoint | None = ...,
        external_tcp: global___EndPoint | None = ...,
        internal_tcp_uses_tls: builtins.bool = ...,
        external_tcp_uses_tls: builtins.bool = ...,
        last_commit_position: builtins.int = ...,
        writer_checkpoint: builtins.int = ...,
        chaser_checkpoint: builtins.int = ...,
        epoch_position: builtins.int = ...,
        epoch_number: builtins.int = ...,
        epoch_id: shared_pb2.UUID | None = ...,
        node_priority: builtins.int = ...,
        is_read_only_replica: builtins.bool = ...,
        advertise_host_to_client_as: builtins.str = ...,
        advertise_http_port_to_client_as: builtins.int = ...,
        advertise_tcp_port_to_client_as: builtins.int = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["epoch_id", b"epoch_id", "external_tcp", b"external_tcp", "http_end_point", b"http_end_point", "instance_id", b"instance_id", "internal_tcp", b"internal_tcp"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["advertise_host_to_client_as", b"advertise_host_to_client_as", "advertise_http_port_to_client_as", b"advertise_http_port_to_client_as", "advertise_tcp_port_to_client_as", b"advertise_tcp_port_to_client_as", "chaser_checkpoint", b"chaser_checkpoint", "epoch_id", b"epoch_id", "epoch_number", b"epoch_number", "epoch_position", b"epoch_position", "external_tcp", b"external_tcp", "external_tcp_uses_tls", b"external_tcp_uses_tls", "http_end_point", b"http_end_point", "instance_id", b"instance_id", "internal_tcp", b"internal_tcp", "internal_tcp_uses_tls", b"internal_tcp_uses_tls", "is_alive", b"is_alive", "is_read_only_replica", b"is_read_only_replica", "last_commit_position", b"last_commit_position", "node_priority", b"node_priority", "state", b"state", "time_stamp", b"time_stamp", "writer_checkpoint", b"writer_checkpoint"]) -> None: ...

global___MemberInfo = MemberInfo

class ReplicaLogWrite(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LOG_POSITION_FIELD_NUMBER: builtins.int
    REPLICA_ID_FIELD_NUMBER: builtins.int
    log_position: builtins.int
    replica_id: builtins.bytes
    def __init__(
        self,
        *,
        log_position: builtins.int = ...,
        replica_id: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["log_position", b"log_position", "replica_id", b"replica_id"]) -> None: ...

global___ReplicaLogWrite = ReplicaLogWrite

class ReplicatedTo(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LOG_POSITION_FIELD_NUMBER: builtins.int
    log_position: builtins.int
    def __init__(
        self,
        *,
        log_position: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["log_position", b"log_position"]) -> None: ...

global___ReplicatedTo = ReplicatedTo

class Epoch(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    EPOCH_POSITION_FIELD_NUMBER: builtins.int
    EPOCH_NUMBER_FIELD_NUMBER: builtins.int
    EPOCH_ID_FIELD_NUMBER: builtins.int
    epoch_position: builtins.int
    epoch_number: builtins.int
    epoch_id: builtins.bytes
    def __init__(
        self,
        *,
        epoch_position: builtins.int = ...,
        epoch_number: builtins.int = ...,
        epoch_id: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["epoch_id", b"epoch_id", "epoch_number", b"epoch_number", "epoch_position", b"epoch_position"]) -> None: ...

global___Epoch = Epoch

class SubscribeReplica(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LOG_POSITION_FIELD_NUMBER: builtins.int
    CHUNK_ID_FIELD_NUMBER: builtins.int
    LASTEPOCHS_FIELD_NUMBER: builtins.int
    IP_FIELD_NUMBER: builtins.int
    PORT_FIELD_NUMBER: builtins.int
    LEADER_ID_FIELD_NUMBER: builtins.int
    SUBSCRIPTION_ID_FIELD_NUMBER: builtins.int
    IS_PROMOTABLE_FIELD_NUMBER: builtins.int
    VERSION_FIELD_NUMBER: builtins.int
    log_position: builtins.int
    chunk_id: builtins.bytes
    @property
    def LastEpochs(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Epoch]: ...
    ip: builtins.bytes
    port: builtins.int
    leader_id: builtins.bytes
    subscription_id: builtins.bytes
    is_promotable: builtins.bool
    version: builtins.int
    def __init__(
        self,
        *,
        log_position: builtins.int = ...,
        chunk_id: builtins.bytes = ...,
        LastEpochs: collections.abc.Iterable[global___Epoch] | None = ...,
        ip: builtins.bytes = ...,
        port: builtins.int = ...,
        leader_id: builtins.bytes = ...,
        subscription_id: builtins.bytes = ...,
        is_promotable: builtins.bool = ...,
        version: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["LastEpochs", b"LastEpochs", "chunk_id", b"chunk_id", "ip", b"ip", "is_promotable", b"is_promotable", "leader_id", b"leader_id", "log_position", b"log_position", "port", b"port", "subscription_id", b"subscription_id", "version", b"version"]) -> None: ...

global___SubscribeReplica = SubscribeReplica

class ReplicaSubscriptionRetry(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LEADER_ID_FIELD_NUMBER: builtins.int
    SUBSCRIPTION_ID_FIELD_NUMBER: builtins.int
    leader_id: builtins.bytes
    subscription_id: builtins.bytes
    def __init__(
        self,
        *,
        leader_id: builtins.bytes = ...,
        subscription_id: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["leader_id", b"leader_id", "subscription_id", b"subscription_id"]) -> None: ...

global___ReplicaSubscriptionRetry = ReplicaSubscriptionRetry

class ReplicaSubscribed(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LEADER_ID_FIELD_NUMBER: builtins.int
    SUBSCRIPTION_ID_FIELD_NUMBER: builtins.int
    SUBSCRIPTION_POSITION_FIELD_NUMBER: builtins.int
    leader_id: builtins.bytes
    subscription_id: builtins.bytes
    subscription_position: builtins.int
    def __init__(
        self,
        *,
        leader_id: builtins.bytes = ...,
        subscription_id: builtins.bytes = ...,
        subscription_position: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["leader_id", b"leader_id", "subscription_id", b"subscription_id", "subscription_position", b"subscription_position"]) -> None: ...

global___ReplicaSubscribed = ReplicaSubscribed

class ReplicaLogPositionAck(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SUBSCRIPTION_ID_FIELD_NUMBER: builtins.int
    REPLICATION_LOG_POSITION_FIELD_NUMBER: builtins.int
    WRITER_LOG_POSITION_FIELD_NUMBER: builtins.int
    subscription_id: builtins.bytes
    replication_log_position: builtins.int
    writer_log_position: builtins.int
    def __init__(
        self,
        *,
        subscription_id: builtins.bytes = ...,
        replication_log_position: builtins.int = ...,
        writer_log_position: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["replication_log_position", b"replication_log_position", "subscription_id", b"subscription_id", "writer_log_position", b"writer_log_position"]) -> None: ...

global___ReplicaLogPositionAck = ReplicaLogPositionAck

class CreateChunk(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LEADER_ID_FIELD_NUMBER: builtins.int
    SUBSCRIPTION_ID_FIELD_NUMBER: builtins.int
    CHUNK_HEADER_BYTES_FIELD_NUMBER: builtins.int
    FILE_SIZE_FIELD_NUMBER: builtins.int
    IS_COMPLETED_CHUNK_FIELD_NUMBER: builtins.int
    leader_id: builtins.bytes
    subscription_id: builtins.bytes
    chunk_header_bytes: builtins.bytes
    file_size: builtins.int
    is_completed_chunk: builtins.bool
    def __init__(
        self,
        *,
        leader_id: builtins.bytes = ...,
        subscription_id: builtins.bytes = ...,
        chunk_header_bytes: builtins.bytes = ...,
        file_size: builtins.int = ...,
        is_completed_chunk: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["chunk_header_bytes", b"chunk_header_bytes", "file_size", b"file_size", "is_completed_chunk", b"is_completed_chunk", "leader_id", b"leader_id", "subscription_id", b"subscription_id"]) -> None: ...

global___CreateChunk = CreateChunk

class RawChunkBulk(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LEADER_ID_FIELD_NUMBER: builtins.int
    SUBSCRIPTION_ID_FIELD_NUMBER: builtins.int
    CHUNK_START_NUMBER_FIELD_NUMBER: builtins.int
    CHUNK_END_NUMBER_FIELD_NUMBER: builtins.int
    RAW_POSITION_FIELD_NUMBER: builtins.int
    RAW_BYTES_FIELD_NUMBER: builtins.int
    COMPLETE_CHUNK_FIELD_NUMBER: builtins.int
    leader_id: builtins.bytes
    subscription_id: builtins.bytes
    chunk_start_number: builtins.int
    chunk_end_number: builtins.int
    raw_position: builtins.int
    raw_bytes: builtins.bytes
    complete_chunk: builtins.bool
    def __init__(
        self,
        *,
        leader_id: builtins.bytes = ...,
        subscription_id: builtins.bytes = ...,
        chunk_start_number: builtins.int = ...,
        chunk_end_number: builtins.int = ...,
        raw_position: builtins.int = ...,
        raw_bytes: builtins.bytes = ...,
        complete_chunk: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["chunk_end_number", b"chunk_end_number", "chunk_start_number", b"chunk_start_number", "complete_chunk", b"complete_chunk", "leader_id", b"leader_id", "raw_bytes", b"raw_bytes", "raw_position", b"raw_position", "subscription_id", b"subscription_id"]) -> None: ...

global___RawChunkBulk = RawChunkBulk

class DataChunkBulk(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LEADER_ID_FIELD_NUMBER: builtins.int
    SUBSCRIPTION_ID_FIELD_NUMBER: builtins.int
    CHUNK_START_NUMBER_FIELD_NUMBER: builtins.int
    CHUNK_END_NUMBER_FIELD_NUMBER: builtins.int
    SUBSCRIPTION_POSITION_FIELD_NUMBER: builtins.int
    DATA_BYTES_FIELD_NUMBER: builtins.int
    COMPLETE_CHUNK_FIELD_NUMBER: builtins.int
    leader_id: builtins.bytes
    subscription_id: builtins.bytes
    chunk_start_number: builtins.int
    chunk_end_number: builtins.int
    subscription_position: builtins.int
    data_bytes: builtins.bytes
    complete_chunk: builtins.bool
    def __init__(
        self,
        *,
        leader_id: builtins.bytes = ...,
        subscription_id: builtins.bytes = ...,
        chunk_start_number: builtins.int = ...,
        chunk_end_number: builtins.int = ...,
        subscription_position: builtins.int = ...,
        data_bytes: builtins.bytes = ...,
        complete_chunk: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["chunk_end_number", b"chunk_end_number", "chunk_start_number", b"chunk_start_number", "complete_chunk", b"complete_chunk", "data_bytes", b"data_bytes", "leader_id", b"leader_id", "subscription_id", b"subscription_id", "subscription_position", b"subscription_position"]) -> None: ...

global___DataChunkBulk = DataChunkBulk

class FollowerAssignment(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LEADER_ID_FIELD_NUMBER: builtins.int
    SUBSCRIPTION_ID_FIELD_NUMBER: builtins.int
    leader_id: builtins.bytes
    subscription_id: builtins.bytes
    def __init__(
        self,
        *,
        leader_id: builtins.bytes = ...,
        subscription_id: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["leader_id", b"leader_id", "subscription_id", b"subscription_id"]) -> None: ...

global___FollowerAssignment = FollowerAssignment

class CloneAssignment(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LEADER_ID_FIELD_NUMBER: builtins.int
    SUBSCRIPTION_ID_FIELD_NUMBER: builtins.int
    leader_id: builtins.bytes
    subscription_id: builtins.bytes
    def __init__(
        self,
        *,
        leader_id: builtins.bytes = ...,
        subscription_id: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["leader_id", b"leader_id", "subscription_id", b"subscription_id"]) -> None: ...

global___CloneAssignment = CloneAssignment

class DropSubscription(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LEADER_ID_FIELD_NUMBER: builtins.int
    SUBSCRIPTION_ID_FIELD_NUMBER: builtins.int
    leader_id: builtins.bytes
    subscription_id: builtins.bytes
    def __init__(
        self,
        *,
        leader_id: builtins.bytes = ...,
        subscription_id: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["leader_id", b"leader_id", "subscription_id", b"subscription_id"]) -> None: ...

global___DropSubscription = DropSubscription



from __future__ import annotations

import enum
import json
from dataclasses import dataclass
from typing import Mapping, Optional, Type, TypeVar, Union

from esdb.generated.persistent_pb2 import CreateReq, ReadReq, ReadResp
from esdb.generated.persistent_pb2 import SubscriptionInfo as SubscriptionInfoPB
from esdb.generated.persistent_pb2 import UpdateReq
from esdb.streams.types import ContentType


@dataclass
class Event:
    id: str
    retry_count: int
    stream: str
    prepare_position: int
    commit_position: int
    metadata: Mapping[str, str]
    type: str
    data: Union[bytes, dict]

    @staticmethod
    def from_read_response_event(event: ReadResp.ReadEvent) -> Event:
        return Event(
            id=event.event.id.string,
            retry_count=event.retry_count,
            stream=event.event.stream_identifier.stream_name.decode(),
            prepare_position=event.event.prepare_position,
            commit_position=event.event.commit_position,
            metadata=event.event.metadata,
            type=event.event.metadata["type"],
            data=json.loads(event.event.data)
            if event.event.metadata["content-type"] == ContentType.JSON.value
            else event.event.data,
        )


SettingsType = TypeVar("SettingsType", CreateReq.Settings, UpdateReq.Settings)


@dataclass
class SubscriptionSettings:
    class ConsumerStrategy(enum.Enum):
        DISPATCH_TO_SINGLE = "DispatchToSingle"
        ROUND_ROBIN = "RoundRobin"
        PINNED = "Pinned"

    live_buffer_size: int
    read_batch_size: int
    history_buffer_size: int
    checkpoint_ms: int
    resolve_links: Optional[bool] = None
    extra_statistics: Optional[bool] = None
    max_retry_count: Optional[int] = None
    min_checkpoint_count: Optional[int] = None
    max_checkpoint_count: Optional[int] = None
    max_subscriber_count: Optional[int] = None
    message_timeout_ms: Optional[int] = None
    consumer_strategy: Optional[ConsumerStrategy] = None

    def to_protobuf(self, cls: Type[SettingsType]) -> SettingsType:
        assert (
            self.read_batch_size < self.live_buffer_size
        ), "read_batch_size may not be greater than or equal to live_buffer_size"
        assert (
            self.read_batch_size < self.history_buffer_size
        ), "read_batch_size may not be greater than or equal to history_buffer_size"
        settings = cls(
            live_buffer_size=self.live_buffer_size,
            read_batch_size=self.read_batch_size,
            history_buffer_size=self.history_buffer_size,
        )

        if self.resolve_links is not None:
            settings.resolve_links = self.resolve_links
        if self.extra_statistics is not None:
            settings.extra_statistics = self.extra_statistics
        if self.max_retry_count is not None:
            settings.max_retry_count = self.max_retry_count
        if self.min_checkpoint_count is not None:
            settings.min_checkpoint_count = self.min_checkpoint_count
        if self.max_checkpoint_count is not None:
            settings.max_checkpoint_count = self.max_checkpoint_count
        if self.max_subscriber_count is not None:
            settings.max_subscriber_count = self.max_subscriber_count
        if self.consumer_strategy:
            if cls is CreateReq.Settings:
                settings.consumer_strategy = self.consumer_strategy.value  # type: ignore
            else:
                settings.named_consumer_strategy = {  # type: ignore
                    self.ConsumerStrategy.DISPATCH_TO_SINGLE: 0,
                    self.ConsumerStrategy.ROUND_ROBIN: 1,
                    self.ConsumerStrategy.PINNED: 2,
                }[self.consumer_strategy]

        settings.checkpoint_after_ms = self.checkpoint_ms

        if self.message_timeout_ms:
            settings.message_timeout_ms = self.message_timeout_ms

        return settings


class NackAction(enum.Enum):
    UNKNOWN = ReadReq.Nack.Action.Unknown
    PARK = ReadReq.Nack.Action.Park
    RETRY = ReadReq.Nack.Action.Retry
    SKIP = ReadReq.Nack.Action.Skip
    STOP = ReadReq.Nack.Action.Stop


@dataclass
class ConnectionInfo:
    username: str
    average_items_per_second: int
    total_items: int
    count_since_last_measurement: int
    observed_measurements: dict[str, int]
    available_slots: int
    in_flight_messages: int
    connection_name: str

    @classmethod
    def from_protobuf(cls, info: SubscriptionInfoPB.ConnectionInfo) -> ConnectionInfo:
        return cls(
            username=info.username,
            average_items_per_second=info.average_items_per_second,
            total_items=info.total_items,
            count_since_last_measurement=info.count_since_last_measurement,
            observed_measurements={m.key: m.value for m in info.observed_measurements},
            available_slots=info.available_slots,
            in_flight_messages=info.in_flight_messages,
            connection_name=info.connection_name,
        )


@dataclass
class SubscriptionInfo:
    event_source: str
    group_name: str
    status: str
    connections: list[ConnectionInfo]
    average_per_second: int
    total_items: int
    count_since_last_measurement: int
    last_checkpointed_event_position: str
    last_known_event_position: str
    resolve_link_tos: bool
    start_from: str
    message_timeout_milliseconds: int
    extra_statistics: bool
    max_retry_count: int
    live_buffer_size: int
    buffer_size: int
    read_batch_size: int
    check_point_after_milliseconds: int
    min_check_point_count: int
    max_check_point_count: int
    read_buffer_count: int
    live_buffer_count: int
    retry_buffer_count: int
    total_in_flight_messages: int
    outstanding_messages_count: int
    consumer_strategy: str
    max_subscriber_count: int
    parked_message_count: int

    @classmethod
    def from_protobuf(cls, info: SubscriptionInfoPB) -> SubscriptionInfo:
        return cls(
            event_source=info.event_source,
            group_name=info.group_name,
            status=info.status,
            connections=[ConnectionInfo.from_protobuf(c) for c in info.connections],
            start_from=info.start_from,
            message_timeout_milliseconds=info.message_timeout_milliseconds,
            extra_statistics=info.extra_statistics,
            max_retry_count=info.max_retry_count,
            live_buffer_size=info.live_buffer_size,
            buffer_size=info.buffer_size,
            read_batch_size=info.read_batch_size,
            check_point_after_milliseconds=info.check_point_after_milliseconds,
            min_check_point_count=info.min_check_point_count,
            max_check_point_count=info.max_check_point_count,
            consumer_strategy=info.named_consumer_strategy,
            max_subscriber_count=info.max_subscriber_count,
            average_per_second=info.average_per_second,
            count_since_last_measurement=info.count_since_last_measurement,
            last_checkpointed_event_position=info.last_checkpointed_event_position,
            last_known_event_position=info.last_known_event_position,
            live_buffer_count=info.live_buffer_count,
            outstanding_messages_count=info.outstanding_messages_count,
            parked_message_count=info.parked_message_count,
            read_buffer_count=info.read_buffer_count,
            resolve_link_tos=info.resolve_link_tos,
            retry_buffer_count=info.retry_buffer_count,
            total_in_flight_messages=info.total_in_flight_messages,
            total_items=info.total_items,
        )



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import streams_pb2 as streams__pb2


class StreamsStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Read = channel.unary_stream(
                '/event_store.client.streams.Streams/Read',
                request_serializer=streams__pb2.ReadReq.SerializeToString,
                response_deserializer=streams__pb2.ReadResp.FromString,
                )
        self.Append = channel.stream_unary(
                '/event_store.client.streams.Streams/Append',
                request_serializer=streams__pb2.AppendReq.SerializeToString,
                response_deserializer=streams__pb2.AppendResp.FromString,
                )
        self.Delete = channel.unary_unary(
                '/event_store.client.streams.Streams/Delete',
                request_serializer=streams__pb2.DeleteReq.SerializeToString,
                response_deserializer=streams__pb2.DeleteResp.FromString,
                )
        self.Tombstone = channel.unary_unary(
                '/event_store.client.streams.Streams/Tombstone',
                request_serializer=streams__pb2.TombstoneReq.SerializeToString,
                response_deserializer=streams__pb2.TombstoneResp.FromString,
                )
        self.BatchAppend = channel.stream_stream(
                '/event_store.client.streams.Streams/BatchAppend',
                request_serializer=streams__pb2.BatchAppendReq.SerializeToString,
                response_deserializer=streams__pb2.BatchAppendResp.FromString,
                )


class StreamsServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Read(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Append(self, request_iterator, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Delete(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Tombstone(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def BatchAppend(self, request_iterator, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_StreamsServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Read': grpc.unary_stream_rpc_method_handler(
                    servicer.Read,
                    request_deserializer=streams__pb2.ReadReq.FromString,
                    response_serializer=streams__pb2.ReadResp.SerializeToString,
            ),
            'Append': grpc.stream_unary_rpc_method_handler(
                    servicer.Append,
                    request_deserializer=streams__pb2.AppendReq.FromString,
                    response_serializer=streams__pb2.AppendResp.SerializeToString,
            ),
            'Delete': grpc.unary_unary_rpc_method_handler(
                    servicer.Delete,
                    request_deserializer=streams__pb2.DeleteReq.FromString,
                    response_serializer=streams__pb2.DeleteResp.SerializeToString,
            ),
            'Tombstone': grpc.unary_unary_rpc_method_handler(
                    servicer.Tombstone,
                    request_deserializer=streams__pb2.TombstoneReq.FromString,
                    response_serializer=streams__pb2.TombstoneResp.SerializeToString,
            ),
            'BatchAppend': grpc.stream_stream_rpc_method_handler(
                    servicer.BatchAppend,
                    request_deserializer=streams__pb2.BatchAppendReq.FromString,
                    response_serializer=streams__pb2.BatchAppendResp.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'event_store.client.streams.Streams', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class Streams(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Read(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/event_store.client.streams.Streams/Read',
            streams__pb2.ReadReq.SerializeToString,
            streams__pb2.ReadResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Append(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_unary(request_iterator, target, '/event_store.client.streams.Streams/Append',
            streams__pb2.AppendReq.SerializeToString,
            streams__pb2.AppendResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Delete(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/event_store.client.streams.Streams/Delete',
            streams__pb2.DeleteReq.SerializeToString,
            streams__pb2.DeleteResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Tombstone(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/event_store.client.streams.Streams/Tombstone',
            streams__pb2.TombstoneReq.SerializeToString,
            streams__pb2.TombstoneResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def BatchAppend(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_stream(request_iterator, target, '/event_store.client.streams.Streams/BatchAppend',
            streams__pb2.BatchAppendReq.SerializeToString,
            streams__pb2.BatchAppendResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['esdb', 'esdb.generated', 'esdb.streams', 'esdb.subscriptions']

package_data = \
{'': ['*']}

install_requires = \
['grpcio>=1.50.0,<2.0.0', 'protobuf>=4.21.8,<5.0.0']

setup_kwargs = {
    'name': 'esdb',
    'version': '0.3.1',
    'description': 'gRPC client for EventStore DB',
    'long_description': '# esdb-py\n\n[![PyPI version](https://badge.fury.io/py/esdb.svg)](https://pypi.org/project/esdb/)\n[![codecov](https://codecov.io/gh/andriykohut/esdb-py/branch/main/graph/badge.svg?token=YVDPTDBPFB)](https://codecov.io/gh/andriykohut/esdb-py)\n\n## EventStoreDB Python gRPC client\n\n> NOTE: This project is still work in progress\n\n<!-- TOC -->\n* [Completed features](#completed-features)\n* [Installation](#installation)\n* [Development](#development)\n* [Usage](#usage)\n  * [Connection string](#connection-string)\n  * [Discovery and node preferences](#discovery-and-node-preferences)\n  * [Connection configuration](#connection-configuration)\n  * [Append, Read, Catch-up subscriptions](#append-read-catch-up-subscriptions)\n  * [Batch append](#batch-append)\n  * [Catch-up subscription to all events with filtering](#catch-up-subscription-to-all-events-with-filtering)\n  * [Persistent subscriptions](#persistent-subscriptions)\n<!-- TOC -->\n\n## Completed features\n\n* [x] secure connection\n* [x] basic auth\n* [x] connection string parsing\n* [x] streams\n  * [x] append\n  * [x] batch append (v21.10+)\n  * [x] delete\n  * [x] read stream\n  * [x] read all with stream/event type filters (v21.10+)\n  * [x] catch-up subscriptions\n  * [x] tombstone\n  * [x] filtering\n* [x] persistent subscriptions\n  * [x] create\n  * [x] read stream\n  * [x] read all with filter (v21.10+)\n  * [x] update\n  * [x] delete\n  * [x] list\n  * [x] info\n  * [ ] reply parked events\n* [ ] CRUD for projections\n* [ ] users\n\n## Installation\n\nUsing pip:\n\n```sh\npip install esdb\n```\n\nUsing poetry:\n\n```sh\npoetry add esdb\n```\n\n## Development\n\n1. Install [poetry](https://python-poetry.org/docs/#installation)\n2. Create virtualenv (i.e. using pyenv):\n\n    ```sh\n    pyenv install 3.10.5\n    pyenv virtualenv 3.10.5 esdb-py\n    pyenv local esdb-py\n    ```\n\n3. Install deps with `poetry install`\n4. Start eventstore in docker: `make run-esdb`\n5. Run the tests: `pytest tests`\n\n## Usage\n\nHave a look at [tests](https://github.com/andriykohut/esdb-py/tree/main/tests) for more examples.\n\n### Connection string examples\n\nDNS discovery with credentials, discovery configuration, node preference and ca file path\n```\nesdb+discover://admin:changeit@localhost:2111?discoveryInterval=0&maxDiscoverAttempts=3&tlsCafile=certs/ca/ca.crt&nodePreference=follower\n```\n\nSingle-node insecure connection\n```\nesdb://localhost:2111?tls=false\n```\n\nSupported parameters:\n - `discoveryInterval`\n - `gossipTimeout`\n - `maxDiscoverAttempts`\n - `nodePreference`\n - `keepAliveInterval`\n - `keepAliveTimeout`\n - `tls`\n - `tlsCafile`\n - `tlsVerifyCert`\n - `defaultDeadline`\n\n\nConnection string can be generated [here](https://developers.eventstore.com/clients/grpc/#connection-details).\n\n### Discovery and node preferences\n\n```py\nfrom esdb import ESClient\n\nclient = ESClient("esdb+discover://admin:changeit@localhost:2111?nodePreference=follower")\n\n```\n\n### Connection configuration\n\n```py\nfrom esdb import ESClient\n\n# Connect without TLS\nclient = ESClient("esdb://localhost:2111?tls=false")\n\n# Secure connection with basic auth and keepalive\nclient = ESClient("esdb://admin:changeit@localhost:2111?tlsCafile=certs/ca/ca.crt&keepAliveInterval=5&keepAliveTimeout=5")\n```\n\n### Append, Read, Catch-up subscriptions\n\n```py\nimport asyncio\nimport datetime\nimport uuid\n\nfrom esdb import ESClient\n\n\nclient = ESClient("esdb+discover://admin:changeit@localhost:2111")\nstream = f"test-{str(uuid.uuid4())}"\n\n\nasync def streams():\n    async with client.connect() as conn:\n        # Appending to stream\n        for i in range(10):\n            append_result = await conn.streams.append(\n                stream=stream,\n                event_type="test_event",\n                data={"i": i, "ts": datetime.datetime.utcnow().isoformat()},\n            )\n\n        # Read up to 10 events\n        async for result in conn.streams.read(stream=stream, count=10):\n            print(result.data)\n\n        # Read up to 10 events, backwards\n        async for result in conn.streams.read(stream=stream, count=10, backwards=True):\n            print(result.data)\n\n        # Read up to 10 events, starting from 5th event\n        async for result in conn.streams.read(stream=stream, count=10, revision=5):\n            print(result.data)\n\n        # Read up to 10 events backwards, starting from 5th event\n        async for result in conn.streams.read(stream=stream, count=10, backwards=True, revision=5):\n            print(result.data)\n\n        # Create a catch-up subscription to a stream\n        async for result in conn.streams.read(stream=stream, subscribe=True):\n            print(result.data)\n\n\nasyncio.run(streams())\n```\n\n### Batch append\n\n```py\nimport asyncio\nimport uuid\n\nfrom esdb import ESClient\nfrom esdb.streams import Message\n\n\nasync def batch_append():\n    # Append multiple events in as a single batch\n    # Batch append is not supported on EventStore < v21.10\n    stream = str(uuid.uuid4())\n    messages: list[Message] = [\n        Message(event_type="one", data={"item": 1}),\n        Message(event_type="one", data={"item": 2}),\n        Message(event_type="one", data={"item": 3}),\n        Message(event_type="two", data={"item": 1}),\n        Message(event_type="two", data={"item": 2}),\n        Message(event_type="two", data={"item": 3}),\n    ]\n    async with ESClient("esdb+discover://admin:changeit@localhost:2111").connect() as conn:\n        response = await conn.streams.batch_append(stream=stream, messages=messages)\n        assert response.current_revision == 5\n        events = [e async for e in conn.streams.read(stream=stream, count=50)]\n        assert len(events) == 6\n\n\nasyncio.run(batch_append())\n```\n\n### Catch-up subscription to all events with filtering\n\n```py\nimport uuid\nimport asyncio\n\nfrom esdb import ESClient\nfrom esdb.shared import Filter\n\n\nasync def filters():\n    async with ESClient("esdb+discover://admin:changeit@localhost:2111").connect() as conn:\n        # Append 10 events with the same prefix to random streams\n        for i in range(10):\n            await conn.streams.append(stream=str(uuid.uuid4()), event_type=f"prefix-{i}", data=b"")\n        # subscribe to events from all streams, filtering by event type\n        async for event in conn.streams.read_all(\n                subscribe=True,  # subscribe will wait for events, use count=<n> to read <n> events and stop\n                filter_by=Filter(\n                    kind=Filter.Kind.EVENT_TYPE,\n                    regex="^prefix-",\n                    # Checkpoint only required when subscribe=True, it\'s not needed when using count=<int>\n                    checkpoint_interval_multiplier=1000,\n                ),\n        ):\n            print(event)\n\n\nasyncio.run(filters())\n```\n\n### Persistent subscriptions\n\n```python\nimport asyncio\nfrom esdb import ESClient\nfrom esdb.shared import Filter\nfrom esdb.subscriptions import SubscriptionSettings, NackAction\n\nclient = ESClient("esdb+discover://admin:changeit@localhost:2111")\n\nstream = "stream-foo"\ngroup = "group-bar"\n\n\nasync def persistent():\n    async with client.connect() as conn:\n        # emit some events to the same stream\n        for i in range(50):\n            await conn.streams.append(stream, "foobar", {"i": i})\n\n        # create a stream subscription\n        await conn.subscriptions.create_stream_subscription(\n            stream=stream,\n            group_name=group,\n            settings=SubscriptionSettings(\n                max_subscriber_count=50,\n                read_batch_size=5,\n                live_buffer_size=10,\n                history_buffer_size=10,\n                consumer_strategy=SubscriptionSettings.ConsumerStrategy.ROUND_ROBIN,\n                checkpoint_ms=10000,\n            ),\n        )\n\n        # create subscription to all events with filtering\n        # Only supported on EventStore v21.10+\n        await conn.subscriptions.create_all_subscription(\n            group_name="subscription_group",\n            filter_by=Filter(kind=Filter.Kind.EVENT_TYPE, regex="^some_type$", checkpoint_interval_multiplier=200),\n            settings=SubscriptionSettings(\n                read_batch_size=50,\n                live_buffer_size=100,\n                history_buffer_size=100,\n                max_retry_count=2,\n                checkpoint_ms=20000,\n            ),\n        )\n\n    # read from a subscription\n    async with client.connect() as conn:\n        sub = conn.subscriptions.subscribe(stream=stream, group_name=group, buffer_size=5)\n        async for event in sub:\n            try:\n                # do work with event\n                print(event)\n                await sub.ack([event])\n            except Exception as err:\n                await sub.nack([event], NackAction.RETRY, reason=str(err))\n\n        # get subscription info\n        info = await conn.subscriptions.get_info(group, stream)\n        assert info.group_name == group\n\n        # delete subscription\n        await conn.subscriptions.delete(group, stream)\n        \n        # list subscriptions\n        subs = await conn.subscriptions.list()\n        for sub in subs:\n            print(sub.total_items)\n\n\nasyncio.run(persistent())\n```\n',
    'author': 'Andrii Kohut',
    'author_email': 'kogut.andriy@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/andriykohut/esdb-py',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from typing import cast

from esdb.generated.shared_pb2 import Empty
from esdb.generated.streams_pb2 import AppendResp, ReadResp


class ClientException(Exception):
    ...


class WrongExpectedVersion(ClientException):
    def __init__(self, error: AppendResp.WrongExpectedVersion) -> None:
        expected_revision = cast(str, error.WhichOneof("expected_revision_option"))
        expected_val = getattr(error, expected_revision)
        current_revision = cast(str, error.WhichOneof("current_revision_option"))
        current_val = getattr(error, current_revision)

        expected = (
            expected_revision if isinstance(expected_val, Empty) else f"{expected_revision}={expected_val}"
        ).replace("expected_", "")

        current = (
            current_revision if isinstance(current_val, Empty) else f"{current_revision}={current_val}"
        ).replace("current_", "")

        super().__init__(f"Expected state '{expected}', got '{current}'")


class StreamNotFound(ClientException):
    def __init__(self, error: ReadResp.StreamNotFound) -> None:
        super().__init__(f"Stream '{error.stream_identifier.stream_name.decode()}' not found")


class DiscoveryError(ClientException):
    ...
